package x.awt;

import java.awt.*;
import java.awt.event.*;

/*
Handles all work associated
with double-buffering. Subclasses should put their
painting code in the drawOffscreen(Graphics g) method.

Copyright:
    This class was written by Martin Wattenberg, who has used it
    in many contexts, some of which are not open-source.
    Therefore, for simplicity, he wishes it to be in the
    public domain.
*/

public class BufferPanel extends Panel
{
    private Image offscreenImage;
    private Graphics offscreen;
	private ActionListener actionListener;
    public void addActionListener(ActionListener l) 
    {
	   actionListener = AWTEventMulticaster.add(actionListener, l);
    }
    
    public void removeActionListener(ActionListener l) 
    {
  	   actionListener = AWTEventMulticaster.remove(actionListener, l);
    }
    
    protected void fireActionEvent(ActionEvent event)
    {
        if (actionListener!=null)
            actionListener.actionPerformed(event);
    }

    // create new offscreen image when panel reshaped.
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(x,y,w,h);
        offscreenImage=createImage(w,h);
        offscreen=offscreenImage.getGraphics();
        //drawOffscreen(offscreen);
        redraw();
    }

    public void update(Graphics g) {paint(g);}

    // draw buffer to screen on paint.
    public void paint(Graphics g)
    {
        //System.out.println("paint: "+offscreenImage);
        if (offscreenImage != null)
            g.drawImage(offscreenImage,0,0,null);
    }

    public void setBackground(Color c)
    {
        super.setBackground(c);
        redraw();
    }

    public void setFont(Font f)
    {
        super.setFont(f);
        redraw();
    }

    // repaint buffer, then paint new buffer to screen.
    public void redraw()
    {
        //System.out.println("BufferPanel.redraw: "+offscreenImage);
        if (offscreenImage != null) 
        {
            drawOffscreen(offscreen);
            Toolkit.getDefaultToolkit().sync();
            //repaint();
        }
        repaint();
    }


//////////////////////////////////////////////////
// protected method for use by subclasses.

    // override this method to do your drawing offscreen.
    protected void drawOffscreen(Graphics g) {}
    

    protected Graphics getBufferGraphics()
    {
        return offscreen;
    }

    protected Image getBufferImage()
    {
        return offscreenImage;
    }

    protected void clearBuffer()
    {
        offscreen.setColor(getBackground());
        offscreen.fillRect(0,0,size().width,size().height);
    }


}