package x.awt;

import java.awt.*;

/**
 *
 * An implementation of ColorScale that uses a gradient
 * to define its colors.
 *
 */
public class GradientColorScale implements ColorScale
{
    private double lowValue, highValue;
    private Color[] cachedColors=new Color[512];
    private Color naColor=Color.gray;
    
    public GradientColorScale(Color lowColor, Color highColor, double lowValue, double highValue)
    {
        setRange(lowValue, highValue);        
        int n=cachedColors.length;
        makeBlend(lowColor, highColor, cachedColors, 0, n);
    }
    
    public GradientColorScale(Color lowColor, Color midColor, Color highColor, double lowValue, double highValue)
    {
        setRange(lowValue, highValue);        
        int n=cachedColors.length;
        makeBlend(lowColor, midColor, cachedColors, 0, n/2);
        makeBlend(midColor, highColor, cachedColors, n/2, n-n/2);
    }
    
    public void setRange(double lowValue, double highValue)
    {
        if (lowValue>=highValue)
            throw new IllegalArgumentException("lowValue>=highValue: "+
                lowValue+" >= "+highValue);
                
        this.lowValue=lowValue;
        this.highValue=highValue;
    }
    
    private void makeBlend(Color c1, Color c2, Color[] target, int start, int n)
    {
        for (int i=0; i<n; i++)
        {
            double u=i/(double)(n-1);
            target[i+start]=new Color(
                blend(u, c1.getRed(), c2.getRed()),
                blend(u, c1.getGreen(), c2.getGreen()),
                blend(u, c1.getBlue(), c2.getBlue())
            );
        }
    }
    
    public void setNAColor(Color naColor)
    {
        this.naColor=naColor;
    }
    
    public Color getNAColor()
    {
        return naColor;
    }
    
    private int blend(double u, int a, int b)
    {
        return (int)((1-u)*a+u*b);
    }
    
    public Color color(double x)
    {
        if (Double.isNaN(x))
            return naColor;
            
        int n=cachedColors.length;
        int m=(int)((n-1)*(x-lowValue)/(highValue-lowValue));
        if (m<0) m=0;
        if (m>=n) m=n-1;
        
        return cachedColors[m];
    }
}