package x.awt;

import java.awt.*;
import java.awt.event.*;

/**
 *
 * Object representing a "lens" which can be moved around
 * on the screen.
 *
 */
public class Lens extends MouseAdapter implements MouseMotionListener
{
    private Rectangle bounds;
    private Rectangle moveableRegion;
    private Dimension maxSize=new Dimension(50,50);
    private Color lightColor=Color.lightGray;
    private Color color=Color.gray;
    private Color darkColor=Color.darkGray;
    
    private int smallPad=3, bigPad=16;
    
    private boolean grasped;
    private Point mouseOffset=new Point(0,0);
    
    Lens()
    {
        setBounds(50,50,30,30);
        setMoveableRegion(0,0,200,200);
    }
	
	
    
	private ActionListener actionListener;
	
    
    public void addActionListener(ActionListener l) 
    {
	   actionListener = AWTEventMulticaster.add(actionListener, l);
    }
    
    public void removeActionListener(ActionListener l) 
    {
  	   actionListener = AWTEventMulticaster.remove(actionListener, l);
    }
    
    protected void fireActionEvent(ActionEvent event)
    {
        if (actionListener!=null)
            actionListener.actionPerformed(event);
    }
    
    void setMoveableRegion(int x, int y, int w, int h)
    {
        moveableRegion=new Rectangle(x,y,w,h);
    }
    
    void setBounds(int x, int y, int w, int h)
    {
        bounds=new Rectangle(x,y,w,h);
    }
    
    Rectangle getBounds()
    {
        return bounds;
    }
    
    public void mousePressed(MouseEvent m)
    {
        grasped= inFrame(m.getX(), m.getY());
        if (grasped)
            mouseOffset=new Point(bounds.x-m.getX(), bounds.y-m.getY());
    }
    
    public void mouseReleased(MouseEvent m)
    {
        grasped=false;
    }
    
    public void mouseMoved(MouseEvent m)
    {
    }
    
    public void mouseDragged(MouseEvent m)
    {
        if (!grasped) return;
        bounds.x=m.getX()+mouseOffset.x;
        bounds.y=m.getY()+mouseOffset.y;
        
        bounds.x=Math.max(moveableRegion.x, bounds.x);
        bounds.x=Math.min(bounds.x, moveableRegion.x+moveableRegion.width-bounds.width);
        bounds.y=Math.max(moveableRegion.y+smallPad, bounds.y);
        bounds.y=Math.min(bounds.y, moveableRegion.y+moveableRegion.height-bounds.height-bigPad);
        
        fireActionEvent(new ActionEvent(this,78787,"position-changed"));
    }
    
    void setMaxSize(int w, int h)
    {
        maxSize=new Dimension(w,h);
    }
    
    private boolean inFrame(int a, int b)
    {
        a-=(bounds.x+smallPad);
        b-=(bounds.y+bounds.height);
        return a>=0 && b>=0 && a<bounds.width+2*smallPad && b<=bigPad;
    }
    
    void draw(Graphics g)
    {
        
        int x=bounds.x, y=bounds.y, w=bounds.width, h=bounds.height;
        //g.setColor(new Color(0,51,153,51));
        //g.fillRect(x,y,w,h);
        g.setColor(color);
        g.fillRect(x-smallPad,y+h,w+2*smallPad,bigPad);
        g.fillRect(x-smallPad,y,smallPad,h+smallPad);
        g.fillRect(x+w,y,smallPad,h+smallPad);
        g.fillRect(x-smallPad,y-smallPad,2*smallPad+w,smallPad);
        int dx=0;
        for (int u=x+smallPad; u<x+w-smallPad*2; u+=4)
        {
            for (int v=y+h+smallPad; v<y+h-smallPad+bigPad; v+=3)
            {
                g.setColor(lightColor);
                g.drawLine(u+dx,v,u+dx,v);
                g.setColor(darkColor);
                g.drawLine(u+dx+1,v+1,u+dx+1,v+1);
                dx=2-dx;
            }
            
        }
        bevel(g, x-smallPad,y-smallPad, w+2*smallPad,h+bigPad+smallPad, false);
        bevel(g, x,y,w,h, true);
        
    }
    
    void bevel(Graphics g, int x, int y, int w, int h, boolean reverse)
    {
        g.setColor(reverse ? darkColor : lightColor);
        g.drawLine(x,y,x+w,y);
        g.drawLine(x,y,x,y+h);
        g.setColor(reverse ? lightColor : darkColor);
        g.drawLine(x+w,y,x+w,y+h);
        g.drawLine(x,y+h,x+w,y+h);
    }
}