package x.awt;

import java.awt.*;
import java.util.*;

/**
 *
 * Object for drawing long pieces of text, wrapping the
 * lines appropriately.
 *
 */
public class LongText implements Drawing
{
    private Vector lines=new Vector();
    private int leading;
    private int lineWidth;
    private Font font;
    
    /**
     *
     * Create a new LongText object, using the given object's
     * toString() method to create the text.
     *
     */
    public LongText(Object o, Font font, int lineWidth) 
    {
        this.lineWidth=lineWidth;
        this.font=font;
        FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
        leading=fm.getHeight();
        String s=o.toString();

        StringTokenizer t = new StringTokenizer(s," \n\r\t\f", true);

        String currentLine="";
        int w=0;            // which has zero width

        while(t.hasMoreElements()) 
        {          
            String word=t.nextToken();
            if (" ".equals(word)) continue;
            if ("\n\r\t\f".indexOf(word)!=-1)
            {
                lines.addElement(currentLine);
                currentLine="";
                w=0;
                continue;
            }
            String newWord=currentLine.length()==0 ? word : " "+word;
            int newWordWidth = fm.stringWidth(newWord); // measure word
            w += newWordWidth;
            if (w < lineWidth || w == newWordWidth) // if we've gone beyond boundary in first word,
                    currentLine += newWord;       // best to let it be.
            else 
            {
                lines.addElement(currentLine);
                currentLine=word;
                w = fm.stringWidth(word);
            }
            
        }
        if (currentLine.length()>0)
            lines.addElement(currentLine);
    }
    
    public void setLeading(int leading)
    {
        this.leading=leading;
    }

    public Dimension getSize()
    {
        return new Dimension(lineWidth, leading*lines.size());
    }

    public void draw(Graphics g, int x, int y) 
    {
        g.setFont(font);
        int n=lines.size();
        for (int i=0; i<n; i++) 
        {  
            y+=leading;
            g.drawString((String)lines.elementAt(i),x,y);
        }
    }

}  

