package x.cluster;

import java.io.*;
import java.util.*;

// NOTE: This doesn't seem to read node values correctly,
// but I'm not sure if it should, investigate later??

public class ClusterFormat
{
	static int numErrs=0;
	
    public static void write(ClusterNode c, Object[] data, PrintStream out)
    {
        int n=c.getChildCount();
        if (n==0) return;

        for (int i=0; i<n; i++)
            write(c.getChild(i), data, out);
        
        StringBuffer b=new StringBuffer(100);
        b.append(name(c, data)).append("\t").
          append(name(c.getChild(0), data)).append("\t").
          append(name(c.getChild(1), data)).append("\t").
          append(c.getValue());
        
        out.println(b);
    }
    
    private static String name(ClusterNode c, Object[] data)
    {
        if (c.hasChildren())
            return "NODE"+c.getID()+"X";
        else
        {
            int i=c.getDataIndex();
            if (i<0 || i>=data.length)
                throw new IllegalArgumentException("ClusterFormat reading: Data index of "+c+" doesn't "+
                    "fit data array of length: "+data.length);
            return data[c.getDataIndex()].toString();
        }
    }
    
    public static ClusterNode read(String fileName, Object[] data) throws IOException, 
                                                                  ClusterParseException
    {
        FileReader f=new FileReader(fileName);
        BufferedReader r=new BufferedReader(f);
        ClusterNode c=read(r, data);
        f.close();
        return c;
    }
    
    public static ClusterNode read(BufferedReader in, Object[] data) throws IOException, 
                                                                  ClusterParseException
    {
        Hashtable leafTable=new Hashtable();
        for (int i=0; i<data.length; i++)
            leafTable.put(data[i].toString(), new Integer(i));
        numErrs=0;
        String line=null;
        ClusterNode c=null;
        Hashtable nodeTable=new Hashtable();
        while (null !=(line=in.readLine()))
        {
            StringTokenizer t=new StringTokenizer(line, "\t");
            if (t.countTokens()!=4)
                throw new ClusterParseException("Line doesn't have 4 tab-separated values: "+line);
            String parent=t.nextToken();
            String child1=t.nextToken();
            String child2=t.nextToken();
            float value=0;
            try
            {
                value=Float.valueOf(t.nextToken()).floatValue();
            }
            catch (NumberFormatException n)
            {
                throw new ClusterParseException("Bad value field in this line: "+line);
            }
            
            c=new ClusterNode();
            nodeTable.put(parent, c);
            c.setChildren(
                getNode(line, child1, leafTable, nodeTable),
                getNode(line, child2, leafTable, nodeTable));
        }
        if (numErrs>0)
        {
        	System.err.println("CluserFormat: num errs="+numErrs);
        
        }
		return c;
    }
    
    static ClusterNode getNode(String line, String key, Hashtable leafTable, Hashtable nodeTable) throws ClusterParseException
    {
        ClusterNode c=(ClusterNode)nodeTable.get(key);
        if (c!=null) return c;
        
        Integer num=(Integer)leafTable.get(key);
        if (num==null)
        {
            //throw new ClusterParseException("Unknown node, "+key+", in line: "+line);
        	System.err.println("Unknown node, "+key+", in line: "+line);
        	c=new ClusterNode();
        	c.setError(true);
        	numErrs++;
        	return c;
        }
        
        return new ClusterNode(num.intValue());
    }
}