package x.cluster;

import java.util.*;
import x.util.*;
import x.math.*;

/**
 *
 * Perform single-linkage clustering on the given points in
 * Euclidean space. Note that the algorithm is a little slow
 * but is extremely space efficient.
 *
 */
public class SingleLinkageCluster implements Ordering, HierarchicalClusterAnalysis
{
    private MST treeAlgorithm;
    
    public SingleLinkageCluster(MetricType type)
    {
        if (type==MetricType.Euclidean)
            treeAlgorithm=new EuclideanMST();
        else if (type==MetricType.PearsonCorrelation)
            treeAlgorithm=new DotProductMST();
        else
            throw new IllegalArgumentException("Unknown metric type: "+type);
    }
    
    public ClusterNode makeClusterTree(float[][] data)
    {
        if (data==null)
            throw new NullPointerException("Given null data to cluster");
        if (data.length==0)
            throw new IllegalArgumentException("Given 0-length array to cluster.");
        
        // Create cluster tree.
        ClusterNode root=makeTree(data);
        
        return root;
    }
    
    private ClusterNode makeTree(float[][] data)
    {
        int i,n=data.length;
            
        Edge[] mst=treeAlgorithm.makeMST(data);
        Sort10.sort(this, mst);
        
        ClusterNode[] c=new ClusterNode[n];
        for (i=0; i<n; i++)
        {
            c[i]=new ClusterNode(i, Geometry.sum(data[i]));
        }
            
        for (i=0; i<n-1; i++)
        {
            ClusterNode c1=c[mst[i].v1].getRoot();
            ClusterNode c2=c[mst[i].v2].getRoot();
            ClusterNode cluster=new ClusterNode();
            if (c1.getValue()>c2.getValue())
                cluster.setChildren(c1,c2);
            else
                cluster.setChildren(c2,c1);
        }
        
        return c[0].getRoot();
    }
    
    public int order(Object a, Object b)
    {
        double t=((Edge)a).length-((Edge)b).length;
        if (t<0) return 1;
        if (t>0) return -1;
        return 0;
    }
    
    
    
}