package x.cluster.test;

import x.cluster.*;

import java.applet.*;
import java.awt.*;
import java.util.*;

/**
 *
 * Tests single-linkage clustering.
 *
 */
public class ClusterTest extends x.awt.FrameApp
{
    static int SIZE=50;
    static int DIM=30;
    Edge[] tree;
    float[][] vertex;
    long totalTime;
    ClusterNode root;
    Color[] palette;
    
    public static void main(String[] args)
    {
        try
        {
            SIZE=Integer.parseInt(args[0]);
            DIM=Integer.parseInt(args[1]);
            ClusterTest c=new ClusterTest();
            c.init();
            c.launch(100,100,400,400);
        }
        catch (Exception e)
        {
            e.printStackTrace(System.err);
            System.err.println("usage: ClusterTest size dim");
        }
    }
    
    public void init()
    {
        makeNew();
        palette=new Color[512];
        for (int i=0; i<256; i++)
        {
            palette[255-i]=new Color(i,0,0);
            palette[256+i]=new Color(0,i,0);
        }
    }
    
    Color getColor(double v)
    {
        int n=256+(int)(128*v);
        if (n<0) n=0;
        if (n>511) n=511;
        return palette[n];
    }
    
    public void paint(Graphics g)
    {
        int w=size().width, h=size().height;
        int n=vertex.length;
        int i;
        
        
        int dw=w/(2*DIM);
        int dh=(2*h)/(3*SIZE);
        int y=100;
        paintCluster(g, root, 10, y, dw, dh);
        
        // info.
        g.setColor(Color.black);
        g.drawString("vertices: "+SIZE, 10, 50);
        g.drawString("dimension: "+DIM, 10, 70);
        g.drawString("Total time: "+totalTime, 10, 90);
        
    }
    
    int paintCluster(Graphics g, ClusterNode c, int x, int y, int dw, int dh)
    {
        if (c.hasData())
        {
            int i=c.getDataIndex();
            for (int j=0; j<vertex[i].length; j++)
            {
                x+=dw;
                g.setColor(getColor(vertex[i][j]));
                g.fillRect(x,y,dw,dh);
            }
            return y+dh;
        }
        else
        {
            int a=y;
            y=paintCluster(g, c.getChild(0), x,y,dw,dh);
            int b= paintCluster(g, c.getChild(1), x,y,dw,dh);
            g.setColor(Color.gray);
            int d=b-a;
            g.drawArc(dw*DIM+50-d/2,a,d,d,270,180);
            return b;
        }
    }
    
    void makeNew()
    {
        Random r=new Random();
        int n=SIZE;
        vertex=new float[n][DIM];
        int w=size().width;
        for (int i=0; i<n; i++)
        {
            float f=Math.random()<.5 ? -.5f : .5f;
            for (int j=0; j<DIM; j++)
            {
                vertex[i][j]=f+(float)(r.nextGaussian());
            }
        }
        long t=System.currentTimeMillis();
        root=new SingleLinkageCluster(MetricType.Euclidean).makeClusterTree(vertex);
        totalTime=System.currentTimeMillis()-t;
    }
    
    
}