package x.cluster.view;

import x.cluster.*;
import x.data.*;
import x.awt.*;
import x.math.*;
import x.viz.biplot.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;



class ClusterBiplot extends Panel
{
    private ClusterNode root1, root2;
    int[] order1, order2;
    private DataArray data;
    DensityBiplotViewer viewer;
    
    
    ClusterBiplot(DataArray data, ClusterNode root1, ClusterNode root2) throws Exception
    {
        this.data=data;
        this.root1=root1;
        this.root2=root2;
        
        // Make ordering.
        order1=root1.makeDataIndexList();
        order2=root2.makeDataIndexList();
        int[] inv2=Permutation.invert(order2);
        
        // Make densities.
        double[] density1=makeDensity(order1);
        double[] density2=makeDensity(order2);
        
        // Make points.
        int n=order1.length;
        PlotPoint[] points=new PlotPoint[n];
        for (int i=0; i<n; i++)
            points[i]=new PlotPoint(i, inv2[order1[i]]);
        
        // Make viewer.
        BiplotModel ungapped=new BiplotModel(density1, density2, points);
        
        Vector xGaps=new Vector();
        Vector yGaps=new Vector();
        addGaps(root1, xGaps, 0);
        //addGaps(root2, yGaps, 0);
        BiplotModel model=Gap.addGaps(ungapped, xGaps, yGaps);
        
        viewer=new DensityBiplotViewer(model);
        setLayout(new GridLayout(1,1));
        add(viewer);
    }
    
    static final int minSep=50;
    
    void addGaps(ClusterNode c, Vector v, int offset)
    {
        if (c.countLeaves()<2*minSep) return;
        int n1=c.getChild(0).countLeaves();
        int n2=c.getChild(1).countLeaves();
        int min=Math.min(n1,n2);
        if (min>minSep)
            v.addElement(new Gap(offset+n1-1, (int)(9*Math.sqrt(min/minSep))));
        addGaps(c.getChild(0), v, offset);
        addGaps(c.getChild(1), v, offset+n1);
        
    }
    
    public void start()
    {
        viewer.start();
    }
    
    public void stop()
    {
        viewer.stop();
    }
    
    double[] makeDensity(int[] order)
    {
        int n=order.length;
        double[] d=new double[n];
        for (int i=0; i<n-1; i++)
        {
            int o1=order[i];
            int o2=order[i+1];
            
            if (o1<0 || o2<0) continue;
            
            double a=dist(data.getRow(o1), data.getRow(o2));
            d[i]=Math.max(0, 1-a);
            //if (i<10)
            //    System.out.println(d[i]);
        }
        
        return d;
    }

    
    double dist(DataRow r1, DataRow r2)
    {
        return 1-Geometry.cov(r1.getNumbers(),r2.getNumbers());
    }
    
}
