package x.cluster.view;

import x.cluster.*;
import x.data.*;
import x.path.*;
import x.awt.*;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;

public class ClusterMapApplet extends Applet
{
    String errorMessage=null;
    private OverviewAndDetail display;
    
    public void init()
    {
        try
        {
            String metaFile=getParameter("metafile");
            String clusterFile=getParameter("clusterfile");
            init(metaFile, clusterFile);
        }
        catch (Exception e)
        {
            StringWriter out=new StringWriter();
            
            PrintWriter p=new PrintWriter(out);
            p.println("Alas, an error occurred. Please cut and paste");
            p.println("the error message below into an e-mail to the");
            p.println("author of this program. Thank you!");
            p.println(" ");
            e.printStackTrace(p);
            errorMessage="Unable to load data. Sorry.";
            setLayout(new GridLayout(1,1));
            TextArea t=new TextArea();
            add(t);
            t.setText(out.toString());
        }
    }
    
    public void paint(Graphics g)
    {
        if (errorMessage!=null)
            g.drawString(errorMessage,10,30);
    }
    
    public void init(String metaFile, String clusterFile) throws Exception
    {
        // Read meta data.
        URL mu=new URL(getDocumentBase(), metaFile);
        InputStream uin=mu.openStream();
        BufferedReader in=new BufferedReader(new InputStreamReader(uin));
        MetaData meta=new MetaData(in);
        in.close();
        uin.close();
        
       
        
        // Read data.
        DataArray data=new DataArray(meta);
        URL du=new URL(getDocumentBase(), meta.getFileName());
        uin=du.openStream();
        InputStream netIn=new BufferedInputStream(uin);
        ZipInputStream zip=new ZipInputStream(netIn);
        zip.getNextEntry();
        
        in=new BufferedReader(new InputStreamReader(zip));
        data.read(true, in);
        netIn.close();
        zip.close();
        in.close();
        uin.close();
        
        
        
        // Read clustering.
        URL cu=new URL(getDocumentBase(), clusterFile);
        uin=cu.openStream();
        in=new BufferedReader(new InputStreamReader(uin));
        ClusterNode root=ClusterFormat.read(in, data.getData());
        in.close();
        uin.close();
        
        // Make order.
        int[] order=root.makeDataIndexList();
        
        // Make layout.
        PathLayout path=new HCurve(data.size());
        //path=new HilbertPath(data.size(), 400, 400);
        //int n=data.size();
        //int m=(int)(1+Math.sqrt(n));
        //path=new ReadingPath(data.size(), m,m);
        
        setBackground(Color.white);
                                                     
        display=new OverviewAndDetail(data, root, order, path);
        setLayout(new GridLayout(1,1));
        add(display);
    }
    
    public void start()
    {
        try
        {
            display.render();
        }
        catch (Exception e)
        {
            e.printStackTrace(System.out);
        }
    }
    
    
    
}