package x.cluster.view;

import x.cluster.*;
import x.data.*;
import x.path.*;


import java.awt.*;
import java.awt.event.*;
import java.io.*;


public class ClusterMapDemo extends Frame
{
	private OverviewAndDetail display;
    
	public static void main(String[] args)
	{
		/*
		if (args.length!=2)
		{
			System.err.println("usage: ClusterMap metafile clusterfile");
			System.err.println("...using defaults instead.");
			//return;
			args=new String[2];
			args[0]="c:\\data\\bio\\whitehead\\lung.meta";
			args[1]="c:\\data\\bio\\whitehead\\lung.gtr";
		}
		*/
		args=new String[2];
		args[0]="lung.meta";
		args[1]="lung.gtr";
       
		try
		{
			ClusterMapDemo c=new ClusterMapDemo();
			c.setBounds(50,50,900,550);
			
			c.launch(args[0], args[1]);
			c.setVisible(true);
			c.display.render();
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);
		}
	}

    
    
	private void launch(String metaFile, String clusterFile)
	{
		Window splash=makeSplash(this);

		// Set up window.
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		});
		
		try
		{
			
			// Read data.
			DataArray data=new DataArray(metaFile);
	        
			// Read clustering for this data.
			ClusterNode root=ClusterFormat.read(clusterFile, data.getData());
	        
			// Make order.
			int[] order=root.makeDataIndexList();
	        
			// Make 2D layout.
			PathLayout path=new HCurve(data.size());
			//PathLayout path=new ReadingPath(data.size(),100,100);
	        
			setBackground(Color.white);
			setLayout(new GridLayout(1,1));
	
			// Set up visualization panel.
			display=new OverviewAndDetail(data, root, order, path);
			add(display);
			
		}
		catch (Exception e)
		{
			e.printStackTrace();
			removeAll();
			ByteArrayOutputStream s=new ByteArrayOutputStream();
			e.printStackTrace(new PrintStream(s));
			TextArea t=new TextArea();
			setLayout(new GridLayout(1,1));
			add(t);
			t.setText(s.toString());
		}
		
		splash.setVisible(false);
        
	}
	static int rnd(int n)
	{
		return (int)(n*Math.random());
	}

    
	static Window makeSplash(Frame t)
	{
		final Font plain=new Font("Verdana", Font.PLAIN, 11);
		final Font bold=new Font("Verdana", Font.BOLD, 11);
		final Font bigFont=new Font("Verdana", Font.BOLD, 36);	
		
		
		Window splash=new Window(t)
		{ 
			public void paint(Graphics g)
			{
				int w=getSize().width, h=getSize().height;
				int m=60;
				Color fg=new Color(0x000000);
				Color bg=Color.white;//new Color(235,245,255);
				g.setColor(bg);
				g.fillRect(0,0,w,h);
				
				//g.setColor(fg);
				//g.fillRect(0,0,m,h);
				
				g.setFont(bigFont);
				g.setColor(Color.gray);
				g.drawRect(0,0,w-1,h-1);
				g.drawString("Jigsaw Map Demo", m+5, 40);
				g.setFont(plain);
				g.drawString("Reading data.", m+5, 70);
				g.drawString("This will take just a moment.", m+5, 86);
				Color[] c=new Color[] {
					Color.lightGray,
					new Color(0xcc6666),
					new Color(0x6666cc)
				};
				int s=10;
				for (int x=0; x<40; x++)
					for (int y=0; y<1; y++)
					{
						g.setColor(c[rnd(3)]);
						g.fillRect(x*s+m+5,y*s+100,s-1,s-1);
					}
				g.setFont(bold);
				g.setColor(Color.lightGray);
				g.drawString("v 0.11", 5, h-40);
				g.setColor(Color.gray);
				g.drawString("Credits:",m+5, h-40);
				g.setFont(plain);
				g.drawString("Martin Wattenberg",m+5, h-25);
				g.drawString("Frank David",m+5, h-10);
				//g.drawString("IBM Research",m+5, h-10);
				
				Toolkit.getDefaultToolkit().sync();
			}
		};
		Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
		int sw=500, sh=300;
		splash.setBounds((d.width-sw)/2, (d.height-sh)/2, sw, sh);
		splash.setBackground(new Color(0xfffff3));
		splash.setVisible(true);
		return splash;
	}
}