package x.cluster.view;

import x.cluster.*;
import x.path.*;
import x.awt.*;
import x.data.*;
import x.label.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;


public class ClusterMapDetail extends BufferPanel implements MouseMotionListener
{
    MapInfo info;
    Grid grid;
    Rectangle detailRect;
    Rectangle sourceRect;
    double detailX1, detailY1, detailX2, detailY2;
    Point mouse=new Point(-1000,0);
    int mouseRowNum=-1;
    
    public ClusterMapDetail(final MapInfo info)
    {
        this.info=info;
        addMouseMotionListener(this);
    }
    
    int getMouseRowNum()
    {
    	return mouseRowNum;
    }
    
    public void mouseMoved(MouseEvent m)
    {
        doMouse(m.getX(), m.getY());
    }
    
    public void mouseDragged(MouseEvent m)
    {
        doMouse(m.getX(), m.getY());
    }
    
    public void mouseEntered(MouseEvent m)
    {
        doMouse(m.getX(), m.getY());
    }
    
    public void mouseExited(MouseEvent m)
    {
        doMouse(-1000,0);
    }
    
    private void doMouse(int x, int y)
    {
        mouse=new Point(x,y);
        redraw();
    }
    
    public void setDetailRect(int sx, int sy, int sw, int sh,
                              int dx, int dy, int dw, int dh)
    {
        detailRect=new Rectangle(dx,dy,dw,dh);
        sourceRect=new Rectangle(sx,sy,sw,sh);
        int w=info.path.getSize().width;
        int h=info.path.getSize().height;
        detailX1=w*(dx-sx)/(double)sw;
        detailY1=h*(dy-sy)/(double)sh;
        detailX2=w*(dx+dw-sx)/(double)sw;
        detailY2=h*(dy+dh-sy)/(double)sh;
    }
    
    synchronized public void drawOffscreen(Graphics g)
    {
        clearBuffer();
        
        if (detailRect==null) return;
        int w=getSize().width, h=getSize().height;
        
        int length=info.data.size();
        int i;
        if (info.renderDistances)
        {
            for (i=0; i<length; i++)
            {
                int px=info.path.x(i);
                int py=info.path.y(i);
                if (px>=detailX1-1 && px<=detailX2+1 && 
                    py>=detailY1-1 && py<=detailY2+1)
                {
                
                    connect(g, i, -1, 0,w,h);
                    connect(g, i, 1, 0,w,h);
                    connect(g, i, 0, -1,w,h);
                    connect(g, i, 0, 1,w,h);
                    
                }
            }
        }
        
        int mr=10;
        int r=7;
        DataRow mouseRow=null;
        int mouseRowX=-1, mouseRowY=-1;
        mouseRowNum=-1;
        for (i=0; i<length; i++)
        {
            int px=info.path.x(i);
            int py=info.path.y(i);
            if (px>=detailX1-1 && px<=detailX2+1 && 
                py>=detailY1-1 && py<=detailY2+1)
            {
                int u=info.path.x(i);
                int v=info.path.y(i);
                int a=scaleX(u,w);
                int b=scaleY(v,h);
                
                boolean match=info.renderMatches && info.match[i];
                if (match)
                {
                    g.setColor(info.matchColor);
                    g.fillRect(a-mr, b-mr, 2*mr+1, 2*mr+1);
                }
                
                Color c=info.numColor(i);
                //if (c==null) c=Color.gray;
                if (c!=null)
                {
                    g.setColor(c);//Color.gray);
                    
                    g.fillRect(a-r,b-r,2*r+1,2*r+1);
                }
                else
                    if (!match)
                    {
                        g.setColor(Color.lightGray);
                        g.drawOval(a-r,b-r,2*r,2*r);
                    }
                
                String s=info.annotation(i);
                StringTokenizer t=new StringTokenizer(s," \t;,-:\"'");
                //t.nextToken();
                int ty=1;
                g.setColor(info.textColor);
                //t.nextToken();
                int limit=4;
                boolean doneUnknown=false;
                while (t.hasMoreTokens())
                {
                    String q=t.nextToken().trim();
                    //if ("UNKNOWN".equals(q)/* || "SIMILAR".equals(q)*/) continue;
                    if ("UNKNOWN".equals(q))
                    {
                        if (doneUnknown) continue;
                        doneUnknown=true;
                    }
                    if (q.indexOf("000")!=-1) continue;
                    //if (ty>4) q="...";
                    g.drawString(q, a+4,b+4+12*ty++);
                    if (ty>limit) break;
                }
                
                if (dist(mouse.x, mouse.y, a,b)<mr)
                {
                    mouseRow=info.row(i);
                    mouseRowX=a;
                    mouseRowY=b;
                    mouseRowNum=i;
                }
            }
        }
        
        if (mouseRow!=null)
        {
            Drawing label=info.labeler.labelFor(mouseRow);
            Point corner=BlockLabel.placement(label.getSize(),
                mouseRowX-r,mouseRowY-r,2*r,2*r,
                0,0,w,h);
            label.draw(g, corner.x, corner.y);
            g.setColor(MapInfo.textColor);
            g.drawRect(mouseRowX-r,mouseRowY-r,2*r+1,2*r+1);
            //g.drawRect(a-r-1,b-r-1,2*r+3,2*r+3);
        }
        
        g.setColor(MapInfo.outlineColor);
        g.drawRect(0,0,w-1,h-1);
        
    }
    
    private double dist(double x1, double y1, double x2, double y2)
    {
        return Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
    }
    
    private void connect(Graphics g, int i, int dx, int dy, int w, int h)
    {
        int u=info.path.x(i);
        int v=info.path.y(i);
        int x=u+dx;
        int y=v+dy;
        
        if (info.path.contains(x,y))
        {
            int j=info.path.order(x,y);
            if (i!=j)
            {
                Color c=info.distColor(i,j);
                g.setColor(c);
                int p=(255-c.getRed())/60;
                for (int q=-p; q<=p; q++)
                {
                    int da=q*dy;
                    int db=q*dx;
                    g.drawLine(scaleX(u,w)+da,scaleY(v,h)+db,scaleX(x,w)+da,scaleY(y,h)+db);
                }
            }
        }
    }
    
    int scaleX(int x, int w)
    {
        return (int)(w*(x-detailX1)/(detailX2-detailX1));
    }
    
    int scaleY(int y, int h)
    {
        return (int)(h*(y-detailY1)/(detailY2-detailY1));
    }
    
    
}