package x.cluster.view;

import x.cluster.*;
import x.path.*;
import x.awt.*;
import x.data.*;

import java.awt.*;
import java.awt.event.*;

public class ClusterMapDisplay extends LensPanel
{
    Grid grid;
    ClusterMapDetail detail;
    int ZOOM=1;
    MapInfo info;
    Dimension textMapSize;
    Image textMap;
    
    public ClusterMapDisplay(MapInfo info, ClusterMapDetail detail)
    {
        this.info=info;
        this.detail=detail;
    }
    
    protected void lensMoved(int x, int y, int w, int h)
    {
        detail.setDetailRect(0,0,ZOOM*getSize().width,ZOOM*getSize().height,x,y,w,h);
        detail.redraw();
    }
    
    int zeroCount=0;
    public void xdrawBackdrop(Graphics g)
    {
        // treemap!
        if (g==null) return;
        zeroCount=0;
        drawTree(g, info.root, 0,0,getSize().width, getSize().height);
        System.out.println("zero nodes: "+zeroCount);
    }
    
    void drawTree(Graphics g, ClusterNode node, double x, double y, double w, double h)
    {
        if (node.hasChildren())
        {
            ClusterNode c0=node.getChild(0);
            ClusterNode c1=node.getChild(1);
            int a=c0.countLeaves();
            int b=c1.countLeaves();
            double r=a/(double)(a+b);
            if (w>h)
            {
                drawTree(g,c0,x,y,w*r,h);
                drawTree(g,c1,x+w*r,y,w-w*r,h);
            }
            else
            {
                drawTree(g,c0,x,y,w,h*r);
                drawTree(g,c1,x,y+h*r,w,h-h*r);
            }
            
            g.setColor(info.linkScale.color(.003*node.getDepth()));
            g.drawRect((int)x, (int)y, (int)(x+w)-(int)x, (int)(y+h)-(int)y);
            
            
        }
        else
        {
            if ((int)(x+w)==(int)x || (int)(y+h)==(int)y)
                zeroCount++;
        }
        /*
        else
        {
            //g.setColor(info.numColor(node.getDataIndex()));
            //g.fillRect((int)x, (int)y, (int)(x+w)-(int)x, (int)(y+h)-(int)y);
            g.setColor(Color.blue);
            g.drawRect((int)x, (int)y, (int)(x+w)-(int)x, (int)(y+h)-(int)y);
        }
        */
    }
    
    public void drawBackdrop(Graphics g)
    {
        if (g==null) return;
        
        // Make grid.
        Dimension d2=new Dimension(ZOOM*getSize().width, ZOOM*getSize().height);
        grid=new Grid(d2, info.path.getSize());
        /*
        if (textMapSize==null || !textMapSize.equals(getSize()))
        {
            System.out.println("text testing!");
            if (getSize().width>0 && getSize().height>0)
            {
                System.out.println("decided to draw");
                textMapSize=getSize();
                textMap=createImage(textMapSize.width, textMapSize.height);
                System.out.println("1!");
                Graphics tg=textMap.getGraphics();
                System.out.println("2!");
                drawText(tg);
                System.out.println("text drawn!");
                tg.dispose();
            }
        }
        if (textMapSize!=null)
            g.drawImage(textMap,0,0,null);
        */
        drawConnections(g);
        g.setColor(MapInfo.outlineColor);
        g.drawRect(0,0,getSize().width-1,getSize().height-1);
        
    }
    
    public void drawText(Graphics g)
    {
        new TextMapView(info).draw(g, grid);
    }
    
    public void drawConnections(Graphics g)
    {
        int length=info.data.size();
        int i;
        if (!info.willRenderValues() && info.renderDistances)
        {
            
            for (i=0; i<length; i++)
            {
                connect(g, i, -1, 0);
                connect(g, i, 0, -1);
            }
            
            
        }
        Rectangle r=new Rectangle(0,0,0,0);
        Point pathPoint=new Point(0,0);
        for (i=0; i<length; i++)
        {
            //grid.getScreenCenter(p1, info.path.x(i), info.path.y(i));
            pathPoint.x=info.path.x(i);
            pathPoint.y=info.path.y(i);
            grid.getScreenRect(r, pathPoint);
                
            Color c=info.numColor(i);
            if (c!=null)
            {
                g.setColor(c);
                //g.fillRect(p1.x-1,p1.y-1,3,3);
                g.fillRect(r.x,r.y,r.width,r.height);
            }
            else
            {
                g.setColor(info.dotColor);
                g.drawLine(p1.x, p1.y, p1.x, p1.y);
            }
            
            if (info.renderMatches && info.match[i])
            {
                g.setColor(info.matchColor);
                g.drawRect(r.x,r.y,r.width-1,r.height-1);
                g.drawRect(r.x+1,r.y+1,r.width-3,r.height-3);
            }
            
            
            
            
        }
    }
    
    Point p1=new Point(), p2=new Point();
    Rectangle r1=new Rectangle(), r2=new Rectangle();
    
    private void connectEdges(Graphics g, int i, int dx, int dy)
    {
        
        int x=info.path.x(i)+dx, y=info.path.y(i)+dy;
        
        if (info.path.contains(x,y))
        {
            int j=info.path.order(x,y);
            //Color c=info.distColor(i,j);
            Color c=info.nodeDistColor(i,j);
            if (c!=null)
            {
                g.setColor(c);
                grid.getScreenRect(r1, info.path.x(i), info.path.y(i));
                grid.getScreenRect(r2, x, y);
                
                if (dx==-1)
                    g.drawLine(r1.x, r1.y+1, r1.x, r1.y+r1.height-1);
                else
                    g.drawLine(r1.x+1, r1.y, r1.x+r1.width-1, r1.y);
                /*
                if (dx==0)
                    g.drawLine(mx-2*(my-p1.y),my,mx+2*(my-p1.y), my);
                else
                    g.drawLine(mx,my-2*(mx-p1.x),mx, my+2*(mx-p1.x));
                    */
            }
        }
    }
    
    
    
    
    private void connect(Graphics g, int i, int dx, int dy)
    {
        
        int x=info.path.x(i)+dx, y=info.path.y(i)+dy;
        
        if (info.path.contains(x,y))
        {
            int j=info.path.order(x,y);
            Color c=info.distColor(i,j);
            //Color c=info.significantDistColor(i,j);
            if (c!=null)
            {
                g.setColor(c);
                grid.getScreenCenter(p1, info.path.x(i), info.path.y(i));
                grid.getScreenCenter(p2, x, y);
                g.drawLine(p1.x,p1.y, p2.x,p2.y);
            }
        }
    }
    
}