package x.cluster.view;

import x.cluster.*;
import x.data.*;
import x.path.*;
import x.awt.*;
import x.math.*;

import java.awt.*;

class MapInfo
{
    DataArray data;
    int[] order;
    ColorScale linkScale, valueScale;
    PathLayout path;
    int colorField=-1;
    boolean renderDistances=true;
    boolean renderMatches=true;
    boolean[] match;
    private boolean[] unorderedMatch;
    ClusterNode root;
    int textField=1;
    MakeDataLabel labeler;
    boolean colorByProto=false;
    int proto=0;
    
    
    static Color matchColor=new Color(0x009900);//new Color(0x0044cc);
    static Color bgcolor=Color.black;
    static Color textColor=Color.gray;
    static Color guiTextColor=Color.black;
    static Color guiDisabledColor=Color.gray;
    static Color weakLinkColor=new Color(0xeeeeee);
    static Color strongLinkColor=new Color(0x111111);
    static Color dotColor=new Color(0xcccccc);
    /*
    static Color matchColor=new Color(0x0044cc);
    static Color bgcolor=Color.black;
    static Color textColor=Color.gray;
    static Color guiTextColor=Color.black;
    static Color guiDisabledColor=Color.gray;
    static Color weakLinkColor=new Color(0x111111);
    static Color strongLinkColor=new Color(0xeeeeee);
    static Color dotColor=new Color(0xcccccc);
    */
    
    /*
    static Color lowColor=new Color(0x990000);
    static Color medColor=new Color(0xffffcc);
    static Color highColor=new Color(0x009900);
    */
    /*
    static Color lowColor=new Color(0x00ff00);
    static Color medColor=new Color(0x000000);
    static Color highColor=new Color(0xff0000);
    */
	static Color lowColor=new Color(0x0000cc);
	static Color medColor=new Color(0x0ffffff);
	static Color highColor=new Color(0xcc0000);

    static Color outlineColor=textColor;
    ClusterNode[] indexedList;
    
    MapInfo(DataArray data, ClusterNode root, int[] order, PathLayout path,
            ColorScale linkScale, ColorScale valueScale, MakeDataLabel labeler)
    {
        this.data=data;
        this.root=root;
        this.order=order;
        this.path=path;
        this.linkScale=linkScale;
        this.valueScale=valueScale;
        this.labeler=labeler;
        indexedList=root.makeIndexedList();
        match=new boolean[data.size()];
        unorderedMatch=new boolean[data.size()];
    }
    
    String annotation(int i)
    {
        return row(i).getText()[textField];
    }
    
    int doTextMatch(String target)
    {
    	System.out.println("matching: "+target);
        int n=match.length;
        int i;
        for (i=0; i<n; i++)
            unorderedMatch[i]=false;
        int count= WordFinder.match(data, textField, target, unorderedMatch);
        for (i=0; i<n; i++)
            match[i]=unorderedMatch[order[i]];
		System.out.println("count: "+count);
        return count;
    }
    
    void clearMatches()
    {
        int n=match.length;
        int i;
        for (i=0; i<n; i++)
            match[i]=false;
    }
    
    void setColorField(int colorField)
    {
        this.colorField=colorField;
    }
    
    void setProto(int proto)
    {
    	this.proto=proto;
    }
    
    void setColorByProto(boolean colorByProto)
    {
    	this.colorByProto=colorByProto;
    }
    
    DataRow row(int i)
    {
        return data.getRow(order[i]);
    }
    
    boolean willRenderValues()
    {
        return colorField!=-1;
    }
    
    Color numColor(int rowNum)
    {
    	if (colorByProto)
    	{
    		if (proto==rowNum)
    			return Color.orange;
			double c=Geometry.corr(row(rowNum).getNumbers(),
								   row(proto).getNumbers());
    		return valueScale.color(1-c);
    	}
        if (colorField==-1) return null;
        return valueScale.color(row(rowNum).getNumbers()[colorField]);
    }
    
    Color distColor(int i, int j)
    {
        return linkScale.color(distance(i,j));
    }
    
    Color nodeDistColor(int i, int j)
    {
        return linkScale.color(.003*nodeDistance(i,j));
    }
    
    
    
    
    Color significantDistColor(int i, int j)
    {
        Color c=linkScale.color(distance(i,j));
        if (c.getRed()>0xe0) return null;
        return c;
    }
    
    
    int nodeDistance(int i, int j)
    {
        return indexedList[i].commonAncestorDepth(indexedList[j]);
    }
    
    double distance(int i, int j)
    {
        try
        {
            //double c=Geometry.cov(row(i).getNumbers(),
            //                row(j).getNumbers());
            double c=Geometry.corr(row(i).getNumbers(),
			                row(j).getNumbers());
            return 1-c;
        }
        catch (Exception e)
        {
            System.err.println("i,j="+i+", "+j);
            System.err.println("ord:"+order[i]+", "+order[j]);
            return 1;
        }
    }
    
}
