package x.cluster.view;

import x.cluster.*;
import x.data.*;
import x.path.*;
import x.awt.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;


public class OverviewAndDetail extends Panel 
             implements ItemListener, AdjustmentListener, ActionListener
{
    private Button searchButton, clearButton;
    private TextField searchField;
    private Choice valueChoice;
    private Checkbox linkBox;
    private Hashtable fieldTable=new Hashtable();
    private MapInfo info;
    private Scrollbar linkSlider;
    
    private Label linkLabel;
    private ClusterMapDisplay display;
    private ClusterMapDetail detail;
    //private ColorScale linkScale=
    //    new GradientColorScale(Color.white,new Color(0x333333), 0,.25);
    private ColorScale linkScale=
        //new GradientColorScale(MapInfo.strongLinkColor, MapInfo.weakLinkColor, 0,.25);
        new GradientColorScale(MapInfo.strongLinkColor, MapInfo.weakLinkColor, 0,.15);
    //private ColorScale valueScale=
    //    new GradientColorScale(Color.red,Color.darkGray,Color.green, -2,2);
	private ColorScale valueScale=
		new GradientColorScale(MapInfo.lowColor,MapInfo.medColor,MapInfo.highColor, -.5,.5);
	private ColorScale correlationScale=
		new GradientColorScale(Color.black,Color.lightGray,Color.white, 0,2);
    private Color bgcolor=Color.white;
    
    public OverviewAndDetail(DataArray data, ClusterNode root, int[] order, PathLayout path)
    {
        info=new MapInfo(data, root, order, path, linkScale, valueScale, new ClusterMapLabel());
        
        // Create map displays.
        detail=new ClusterMapDetail(info);
        display=new ClusterMapDisplay(info, detail);
        display.setBackground(bgcolor);
        detail.setBackground(bgcolor);
        detail.setFont(new Font("Helvetica", Font.PLAIN, 9));
		detail.addMouseListener(new MouseAdapter()
		{
			public void mouseClicked(MouseEvent e)
			{
				if (detail.getMouseRowNum()!=-1)
				{
					info.setProto(detail.getMouseRowNum());
					info.setColorByProto(true);
					info.valueScale=correlationScale;
					render();
				}
			}
		});

        
        // Add to panel.
        setLayout(null);
        setBackground(Color.lightGray);
        add(display);
        add(detail);
        
        // Add value choice.
        valueChoice=new Choice();
        valueChoice.addItem("None");
        DataField[] field=data.getFields();
        int count=0;
        for (int i=0; i<field.length; i++)
        {
            int type=field[i].getType();
            String name=field[i].getName();
            if (type==MetaData.NUMBER && name.trim().length()>0)
            {
                count++;
                name=count+". "+name;
                fieldTable.put(name, field[i]);
                valueChoice.addItem(name);
            }
        }
        add(valueChoice);
        valueChoice.addItemListener(this);
        valueChoice.setBounds(0,25,215,20);
        
        Label l=new Label("Color by array:");
        standardAdd(l,0,0,215,20);
        
        linkBox=new Checkbox("Show distances between neighbors", true);
        standardAdd(linkBox, 230,0,215,20);
        linkBox.addItemListener(this);
        
        linkSlider=new Scrollbar(Scrollbar.HORIZONTAL,50,10,0,100);
        add(linkSlider);
        linkSlider.setBounds(290,25,145,20);
        linkSlider.addAdjustmentListener(this);
        linkLabel=new Label("Sensitivity");
        standardAdd(linkLabel,230,25,60,20);
        
        Label sl=new Label("Find (case sensitive):");
        standardAdd(sl, 450,0,215,20);
        searchField=new TextField();
        add(searchField);
        searchField.setBounds(450,25,100,20);
        searchField.setBackground(Color.white);
        searchField.setForeground(Color.black);
        searchField.addActionListener(this);
        
        
        searchButton=new Button("Find");
        add(searchButton);
        searchButton.setBounds(555,25,60,20);
        searchButton.addActionListener(this);
        
        clearButton=new Button("Clear");
        add(clearButton);
        clearButton.setBounds(620,25,60,20);
        clearButton.addActionListener(this);
    }
    
    void standardAdd(Component c, int x, int y, int w, int h)
    {
        c.setBackground(Color.lightGray);
        c.setForeground(Color.black);
        add(c);
        c.setBounds(x,y,w,h);
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource()==searchField || e.getSource()==searchButton)
        {
            info.doTextMatch(searchField.getText());//.toUpperCase());
        }
        if (e.getSource()==clearButton)
        {
            info.clearMatches();
            searchField.setText("");
        }
        render();
    }
    
    public void adjustmentValueChanged(AdjustmentEvent e)
    {
        //System.out.println(e);
        double v=.15*Math.exp(linkSlider.getValue()*.03);
        //System.out.println("v="+v);
        info.linkScale=
            new GradientColorScale(MapInfo.strongLinkColor,
                MapInfo.weakLinkColor, 0,v*.25);
        render();
    }
    
    public void itemStateChanged(ItemEvent e)
    {
        if (e.getSource()==valueChoice)
        {
            DataField d=(DataField)fieldTable.get(valueChoice.getSelectedItem());
            info.setColorField(d==null ? -1 : d.getColumnNum());
            info.setColorByProto(false);
            info.valueScale=valueScale;
        }
        if (e.getSource()==linkBox)
        {
            boolean on=linkBox.getState();
            linkSlider.setEnabled(on);
            linkLabel.setForeground(on ? MapInfo.guiTextColor : MapInfo.guiDisabledColor);
            info.renderDistances=on;
        }
        render();
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        super.setBounds(x,y,w,h);
        int topH=50;
        int side=450;
        display.setBounds(0,topH,side,side);
        detail.setBounds(side,topH,side,side);
    }
    
    public void render()
    {
        display.redrawBackdrop();
        detail.redraw();
    }
}