package x.cluster.view;

import x.label.*;
import x.cluster.*;
import x.data.*;
import x.path.*;

import java.util.*;
import java.awt.*;

class TextMapView
{
    private MapInfo info;
    Color highlightColor=new Color(0xff9900);
    //Color fillColor=new Color(255, 153, 0, 51);
    Color fillColor=new Color(255, 230, 204);
    Color dotColor=new Color(0xffcc99);
    Color lineColor=new Color(255, 245, 225);
    Color textColor=new Color(0x996600);
    Hashtable words;
    ClusterNode[] nodes;
    FontMetrics fm;
    Font font=new Font("Helvetica", Font.PLAIN, 10);
    int[] inverseOrder;
    
    TextMapView(MapInfo info)
    {
        this.info=info;
        System.out.println("textmapview created");
        findTextClusters();
        System.out.println("clusters found");
        fm=Toolkit.getDefaultToolkit().getFontMetrics(font);
    }
    
    void findTextClusters()
    {
        words=WordFinder.getWordFrequency(info.data, info.textField);
        // find bounds for clusters.
        
        int n=info.order.length;
        nodes=new ClusterNode[n];
        inverseOrder=new int[n];
        for (int i=0; i<n; i++)
        {
            inverseOrder[info.order[i]]=i;
        }
        
        locateChildren(info.root, inverseOrder);
        info.root.findBoundsFromChildren();
        
    }
    
    void locateChildren(ClusterNode c, int[] inverseOrder)
    {
        if (!c.hasChildren())
        {
            int i=c.getDataIndex();
            int n=inverseOrder[i];
            c.includeInBounds(info.path.x(n), info.path.y(n));
            nodes[n]=c;
        }
        else
        {
            locateChildren(c.getChild(0), inverseOrder);
            locateChildren(c.getChild(1), inverseOrder);
        }
    }
    
    
    
    void draw(Graphics g, Grid grid)
    {
        g.setFont(font);
        Vector v=new Vector();
        
        int n=info.data.size();
        System.out.println("------ about to enumerate");
        Enumeration e=words.keys();
        while (e.hasMoreElements())
        {
            String s=(String)e.nextElement();
            Integer q=(Integer)words.get(s);
            
            if (q.intValue()>10)
            {
                //System.out.println("s,q: "+s+": "+q);
                //System.out.println(s);
                draw(g, s, v, grid);
            }
        }
        System.out.println("------ about to draw labels");
        LabelRect.placeNicely(v,600,600);
        drawLabels(v,g);
        Toolkit.getDefaultToolkit().sync();
    }    
    
    
    void draw(Graphics g, String word, Vector v, Grid grid)
    {
        int n=info.data.size();
        int i;
        for (i=0; i<n; i++)
        {
            String s=info.annotation(i);
            boolean found= s!=null && s.indexOf(word)!=-1;
            if (nodes[i]!=null)
                nodes[i].setNumMarked(found ? 1 : 0);
        }
        int total=info.root.countNumMarked();
        info.root.resetSignificance();
        double proportion=total/(double)info.root.getNumDescendants();
        
        markCluster(info.root, proportion);
        drawCluster(g, info.root, grid);
        addLabels(v, info.root, grid, word);
    }
    
    void drawCluster(Graphics g, ClusterNode c, Grid grid)
    {
        if (c.getSignificant())
        {
            //Rectangle r=c.getBounds();
            Rectangle r=c.findBoundsFromMarked();
            if (r!=null)
            {
                int rx=grid.getScreenX(r.x);
                int ry=grid.getScreenY(r.y);
                int rw=grid.getScreenX(1+r.x+r.width)-rx;
                int rh=grid.getScreenY(1+r.y+r.height)-ry;
                
                
                g.setColor(fillColor);
                g.fillRect(rx,ry,rw,rh);
                g.setColor(highlightColor);
                g.drawRect(rx,ry,rw,rh);
            }
            return;
        }
        else
            for (int i=c.getChildCount()-1; i>=0; i--)
                drawCluster(g, c.getChild(i), grid);
    }
    
    void drawLabels(Vector v, Graphics g)
    {
        g.setColor(textColor);
        for (int i=v.size()-1; i>=0; i--)
        {
            LabelRect r=(LabelRect)v.elementAt(i);
            g.drawString(r.getLabel(), r.x, r.y);
        }
    }
    
    void addLabels(Vector v, ClusterNode c, Grid grid, String s)
    {
        if (c.getSignificant())
        {
            Rectangle r=c.getBounds();
            if (r!=null)
            {
                int sw=fm.stringWidth(s);
                int sh=fm.getAscent();
                
                int rx=grid.getScreenX(r.x);
                int ry=grid.getScreenY(r.y);
                
                int x=2+rx;
                int y=ry+sh/2;
                Rectangle q=new Rectangle(
                    rx, 
                    ry, 
                    grid.getScreenX(1+r.x+r.width)-rx, 
                    grid.getScreenY(1+r.y+r.height)
                );
                LabelRect l=new LabelRect(s, q, x,y,sw,sh);
                v.addElement(l);
            }
            return;
        }
        else
            for (int i=c.getChildCount()-1; i>=0; i--)
                addLabels(v, c.getChild(i), grid, s);
        
    }
    
    double markCluster(ClusterNode c, double p)
    {
        int m=c.getChildCount();
        double n=c.getNumDescendants();
        double t=c.getNumMarked();
        if (n<3 || t<=2) return 0;
        double e=n*p;
        double s=(t-e)/Math.sqrt(e*(1-p));
        double minDraw=15;//10;
        
        for (int i=0; i<m; i++)
            minDraw=Math.max(minDraw, markCluster(c.getChild(i),p));
            
        c.setSignificant(s>minDraw);
            
        return Math.max(s, minDraw);
    }
    
}