package x.data;

import java.util.*;
import java.io.*;

public  class MetaData
{
    private String source;
    private String fileName;
    private String idField;
    private Vector textFields=new Vector();
    private Vector ignoredFields=new Vector();
    private Vector comments=new Vector();
    private Vector ignoredRowStarts=new Vector();
    
    public static final int NUMBER=0, TEXT=2, IGNORE=3;
    
    public MetaData(String source, String idField)
    {
        if (source==null)
            throw new NullPointerException("Null source.");
        if (idField==null)
            throw new NullPointerException("Null id field.");
        
        this.source=source;
        this.idField=idField;
    }
    
    public void addTextField(String field)
    {
        textFields.addElement(field);
    }
    
    public void addIgnoredField(String field)
    {
        ignoredFields.addElement(field);
    }
    
    public void addIgnoredRowStart(String prefix)
    {
        ignoredRowStarts.addElement(prefix);
    }
    
    public void addComment(String comment)
    {
        comments.addElement(comment);
    }
    
    public MetaData(File file) throws IOException, 
                                               DataParseException
    {
        FileInputStream in=new FileInputStream(file);
        init(new BufferedReader(new InputStreamReader(in)));
    }
    
    public MetaData(BufferedReader in) throws IOException, 
                                           DataParseException
    {
        init(in);
    }
    
    private void init(BufferedReader in) throws IOException, 
                                           DataParseException
    {
        // read data
        String line=null;
        
        
        try
        {
            while (null!=(line=in.readLine()))
            {
                line=line.trim();
                
                if (line.startsWith("#") || line.length()==0)
                {
                    comments.addElement(line);
                    continue;
                }
                StringTokenizer t=new StringTokenizer(line, ":");
                if (t.countTokens()<=1)
                    throw new DataParseException("line not in form 'a:b' --> "+line);
            
                String key=t.nextToken();
                String value=line.substring(key.length()+1);
                
                if ("id".equals(key))
                {
                    if (idField!=null)
                        throw new DataParseException("Multiple id fields.");
                    idField=value;
                }
                else if ("source".equals(key))
                {
                    if (source!=null)
                        throw new DataParseException("Multiple source lines.");
                    source=value;
                }
                else if ("ignore".equals(key))
                    addIgnoredField(value);
                else if ("ignore-rows-starting".equals(key))
                    ignoredRowStarts.addElement(value);
                else if ("text".equals(key))
                    addTextField(value);
                else if ("file".equals(key))
                {
                    if (fileName!=null)
                        throw new DataParseException("Multiple file lines.");
                    fileName=value;
                }
                else
                    throw new DataParseException("Unrecognized field type: "+key);
            }
            
            if (source==null)
                throw new DataParseException("Couldn't find source line in meta data.");
            if (idField==null)
                throw new DataParseException("Couldn't find id line in meta data.");
        }
        finally
        {
            in.close();
        }
    }
    
    public boolean isIDField(String s)
    {
        return s.equals(idField);
    }
    
    public int getFieldType(String s)
    {
        if (ignoredFields.contains(s))
            return IGNORE;
        if (textFields.contains(s))
            return TEXT;
        return NUMBER;
    }
    
    public String getSource()
    {
        return source;
    }
    
    public String getFileName()
    {
        return fileName;
    }
    
    public String getIDField()
    {
        return idField;
    }
    
    public Vector getTextFields()
    {
        return textFields;
    }
    
    public int getTextFieldCount()
    {
        return textFields.size();
    }
    
    public Vector getIgnoredFields()
    {
        return ignoredFields;
    }
    
    public Vector getComments()
    {
        return comments;
    }
    
    public Vector getIgnoredRowStarts()
    {
        return ignoredRowStarts;
    }
    
    public void print(PrintStream p)
    {
        p.println("source: "+getSource());
        p.println("id: "+getIDField());
        p.println("file: "+getFileName());
        p.println("text: "+getTextFields());
        p.println("ignore: "+getIgnoredFields());
        p.println("comments: "+getComments());
        p.println("ignore-rows-starting: "+ignoredRowStarts);
    }
}