package x.data.seq;

import x.data.*;
import x.path.*;
import x.awt.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;
import java.util.*;

public class CompareSequence extends Frame implements ActionListener
{
    DataArray data;
    AveragePanel overview;
    SequenceDetail detail;
    
    public static void main(String[] args)
    {
        if (args.length!=2)
        {
            System.err.println("usage: CompareSequence metafile seqfile");
            return;
        }
        
        try
        {
            CompareSequence c=new CompareSequence();
            c.setBounds(50,50,1000,500);
            c.setVisible(true);
            c.init(args[0], args[1]);
            c.repaint();
            c.overview.repaint();
            
        }
        catch (Exception e)
        {
            e.printStackTrace(System.err);
        }
    }
    
    public void init(String metaFile, String seqFile) throws Exception
    {
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });
        
        
        
        // Read data.
        data=new DataArray(metaFile);
        
        // Read AnnotatedSequence.
        Sequence[] seq=FASTA.readFromFile(seqFile);
        //AnnotatedSequence[] u=ORF.translate(seq, 500);
        AnnotatedSequence[] u=UTR.translate(seq);
        
        setLayout(null);
        
        // make overview panel.
        overview=new AveragePanel();
        add(overview);
        overview.setBounds(0,0,500,500);
        overview.init(u, data, 10, 500, 500);
        overview.redraw();
        overview.addActionListener(this);
        
        // make details
        detail=new SequenceDetail(data, overview.orderedAnnotatedSequence);
        add(detail);
        detail.setBounds(500,0,500,500);
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if (e instanceof ViewActionEvent)
        {
            ViewActionEvent v=(ViewActionEvent)e;
            detail.setStart(v.x, v.y);
        }
    }
}

class SequenceDetail extends BufferPanel
{
    AnnotatedSequence[] u;
    DataArray data;
    int startX=0, startY;
    Color letterColor=new Color(128,160,220);
    Color mainLetterColor=new Color(0,51,153);
    
    SequenceDetail(DataArray data, AnnotatedSequence[] u)
    {
        this.data=data;
        this.u=u;
        for (int i=0; i<u.length; i++)
        {
            if (u[i]==null)
                System.out.println("null seq at "+i);
        }
    }
    
    void setStart(int startX, int startY)
    {
        this.startX=startX;
        this.startY=startY;
        redraw();
    }
    
    public void drawOffscreen(Graphics g)
    {
        clearBuffer();
        g.setColor(letterColor);
        int y=15;
        int i=startY;
        int len=50;
        //System.out.println("startX="+startX);
        //StringBuffer b=new StringBuffer(len);
        while (y<getSize().height && i<u.length)
        {
            int x=5;
            AnnotatedSequence seq=u[i];
            byte[] c=seq.getSequence().data();
            int index=seq.getDataIndex();
            DataRow row=data.getRow(index);
            
            for (int j=c.length+startX; x<200 && j<c.length; j++)
            {
                g.setColor(c[j]==DNA.A ? mainLetterColor : letterColor);
                g.drawString(DNA.stringFor(c[j]),x,y);
                x+=10;
            }
            
            g.setColor(Color.black);
            g.drawString(index+": "+row.getText()[1], x+10,y);
            y+=14;
            i++;
        }
    }
}

class AveragePanel extends BufferPanel implements MouseMotionListener
{
    AnnotatedSequence[] orderedAnnotatedSequence;
    int condense;
    DataArray data;
    String errorMessage=null;
    AverageSequence[] averages;
    ColorScale p;
    ColorScale values;
    Image img;
    Color windowColor=new Color(255,255,0,128);
    int windowY=100, windowHeight=5, windowX=100, windowW=20;
    int w,h;
    
    AveragePanel()
    {
        addMouseMotionListener(this);
    }
    
    void init(AnnotatedSequence[] u, DataArray data, int condense, int w, int h)
    {
        this.data=data;
        this.condense=condense;
        p=new GradientColorScale(Color.black, Color.white, 0, .5);
        values=new GradientColorScale(Color.red, Color.black, Color.green, -1, 1);
        
        // Make AnnotatedSequence table.
        Hashtable seqTable=new Hashtable();
        int i;
        for (i=0; i<u.length; i++)
            seqTable.put(u[i].getID(), u[i]);
        
        // Make averages.
        int goodcount=0;
        int avgCount=0;
        Vector avgs=new Vector();
        AverageSequence a=new AverageSequence();
        orderedAnnotatedSequence=new AnnotatedSequence[data.size()];
        
        
        for (i=0; i<data.size(); i++)
        {
            DataRow d=data.getRow(i);
            StringTokenizer t=new StringTokenizer(d.getText()[1]);
            String id=t.nextToken();
            AnnotatedSequence m=(AnnotatedSequence)seqTable.get(id);
            if (m!=null)
            {
                if (m.getSequence().length()!=500)
                {
                    System.out.println("Bad length: "+m.getSequence().length());
                    continue;
                }
                m.setDataIndex(i);
                orderedAnnotatedSequence[goodcount]=m;
                a.add(m.getSequence());
                goodcount++;
                avgCount++;
                if (avgCount==condense)
                {
                    avgs.addElement(a);
                    a=new AverageSequence();
                    avgCount=0;
                }
            }
        }
        
        AnnotatedSequence[] temp=orderedAnnotatedSequence;
        orderedAnnotatedSequence=new AnnotatedSequence[goodcount];
        System.arraycopy(temp,0,orderedAnnotatedSequence,0,goodcount);
        
        System.out.println("processed AnnotatedSequences: good="+goodcount+" out of "+u.length);
        int size=avgs.size();
        averages=new AverageSequence[size];
        avgs.copyInto(averages);
        System.out.println("num of avgs: "+averages.length);
        
        this.w=w;
        this.h=h;
        makeImage(w,h);
        redraw();
    }
    
    public void mouseMoved(MouseEvent m)
    {
        return;
    }
    
    public void mouseDragged(MouseEvent m)
    {
        mouse(m, true);
    }
    
    public void mouseExited(MouseEvent m)
    {
        mouse(m, false);
    }
    
    public void mouseEntered(MouseEvent m)
    {
        mouse(m, true);
    }
    
    void mouse(MouseEvent m, boolean draw)
    {
        if (draw)
        {
            windowY=m.getY();
            windowX=m.getX();
            fireActionEvent(new ViewActionEvent(this, windowX-getSize().width, condense*windowY));
        }
        else
            windowY=-1000;
        redraw();
    }
    
    
    public void drawOffscreen(Graphics g)
    {
        if (img!=null)
            g.drawImage(img,0,0,null);
        g.setColor(windowColor);
        g.fillRect(windowX,windowY,windowW,windowHeight);
    }
    
    public void paintImage(Graphics g)
    {
        if (errorMessage!=null)
            g.drawString(errorMessage,10,30);
        else
        {
            float t=.65f;
            for (int y=0; y<averages.length; y++)
            {
                int right=averages[y].length();
                float[] num=data.getRow(y*condense).getNumbers();
                int x;
                /*
                for (x=0; x<num.length; x++)
                {
                    g.setColor(values.color(num[x]));
                    int ax=x+right+10;
                    g.drawLine(ax,y,ax,y);
                }
                */
                g.setColor(Color.black);
                
                for (x=0; x<right; x++)
                {
                    
                    // row dotplot
                    float f=averages[y].data(DNA.A)[x];
                    g.setColor(p.color(.5-f*f));
                    g.drawLine(x,y,x,y);
                    
                    
                }
            }
        }
    }
    
    void makeImage(int w, int h)
    {
        img=createImage(w,h);
        Graphics g=img.getGraphics();
        paintImage(g);
        g.dispose();
    }
    
}

    
class ViewActionEvent extends ActionEvent
{
    int x,y;
    
    ViewActionEvent(Object source, int x, int y)
    {
        super(source, Event.ACTION_EVENT, "VIEW-CHANGE");
        this.y=y;
        this.x=x;
    }
}
