package x.data.seq;

import x.data.*;
import x.path.*;
import x.awt.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;
import java.util.*;

public class MotifValue extends Frame
{
    
    DataArray data;
    SequenceValue v;
    ColorScale color=new GradientColorScale(Color.red, Color.black, 
        Color.green/*new Color(0x006600)*/, -.5, .5);
    AnnotatedSequence[] seq;
    
    public static void main(String[] args)
    {
        if (args.length!=2)
        {
            System.err.println("usage: MotifValue metafile seqfile");
            return;
        }
        
        
        try
        {
            MotifValue c=new MotifValue();
            c.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {System.exit(0);}
            });
            c.setBounds(50,50,500,500);
            c.setVisible(true);
            c.init(args[0], args[1]);
            c.repaint();
        }
        catch (Exception e)
        {
            e.printStackTrace(System.err);
        }
    }
    int start=500;
    
    public MotifValue()
    {
        super("Motif value");
        addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent m)
            {
                System.out.println("mouse pressed");
                start+=500;
                repaint();
            }
        });
    }
    
    public void paint(Graphics g)
    {
        if (v==null || seq==null) return;
        
        System.out.println("seq.length="+seq.length);
        System.out.println("start="+start);
        int a=start;
        int b=start+500;
        for (int i=a; i<b; i++)
        {
            try
            {
            Sequence s=seq[i].getSequence();
            for (int j=0; j<494; j++)
            {
                g.setColor(color.color(v.subsequenceValue(s,j,1000)));
                g.drawLine(j,i-a,j,i-a);
            }
            }
            catch (Exception e) {}
        }
        
        g.setColor(Color.white);
        g.drawString("Showing 500 bp UTRs for genes #"+a+" to "+(b-1), 20,50);
        
        /*
        // paint chart of motif values.
        int n=v.getNumSequences();
        int side=(int)Math.round(Math.sqrt(n));
        int u=getSize().width/side;
        for (int i=0; i<n; i++)
        {
            g.setColor(color.color(v.average(i)));
            int x=(i%side)*u;
            int y=(i/side)*u;
            g.fillRect(x,y,u,u);
        }
        */
    }
    
    public void init(String metaFile, String seqFile) throws Exception
    {
        int i;
        
        
        // Read data.
        data=new DataArray(metaFile);
        
        // Read basic sequences.
        Sequence[] unordered=FASTA.readFromFile(seqFile);
        //for (i=0; i<unordered.length; i++)
        //    unordered[i]=unordered[i].first(500);
        
        // Translate to marked sequences.
        AnnotatedSequence[] u=UTR.translate(unordered);
        
        
        // Make AnnotatedSequence table.
        Hashtable seqTable=new Hashtable();
        for (i=0; i<u.length; i++)
            seqTable.put(u[i].getID(), u[i]);
        
        // Make averages.
        int goodcount=0;
        seq=new AnnotatedSequence[data.size()];
        
        // Correlate with data.
        for (i=0; i<data.size(); i++)
        {
            DataRow d=data.getRow(i);
            StringTokenizer t=new StringTokenizer(d.getText()[1]);
            String id=t.nextToken();
            AnnotatedSequence m=(AnnotatedSequence)seqTable.get(id);
            if (m!=null)
            {
                m.setDataIndex(i);
                seq[goodcount]=m;
                goodcount++;
            }
        }
        
        AnnotatedSequence[] temp=seq;
        seq=new AnnotatedSequence[goodcount];
        System.arraycopy(temp,0,seq,0,goodcount);
        System.out.println("processed AnnotatedSequences: good="+goodcount+" out of "+u.length);
        
        int length=6;
        
        findValues(seq, data, length, 1);
    }
    
    void findValues(AnnotatedSequence[] seq, DataArray data, int length, int column)
    {
        v=new SequenceValue(length);
        int n=v.getNumSequences();
        System.out.println("motif length="+length+".  possible seq ="+n);

        int actualTotal=0;
        for (int i=0; i<seq.length; i++)
        {
            actualTotal+=seq[i].length()-length+1;
            int d=seq[i].getDataIndex();
            v.include(seq[i].getSequence(), data.getRow(d).getNumbers()[column]);
            if (i%200==0)
                System.out.print(".");
        }
        
        System.out.println("\n\nDone.");
    }
}

    
