package x.data.seq;

// this is cool, you give it a bunch of sequences,
// each with an associated value, and it makes
// average values for all motifs of a 
// given length.

// It currently only works for small motifs,
// since it creates an array for all possible
// sequences.

public class SequenceValue
{
    public int[] count;
    public float[] value;
    private int length;
    private int numSequences;
    
    public SequenceValue(int length)
    {
        this.length=length;
        numSequences=sizeOf(length);
        count=new int[numSequences];
        value=new float[numSequences];
    }
    
    public void include(Sequence seq, float x)
    {
        int n=seq.length();
        for (int i=0; i<=n-length; i++)
            include(seq.data(),i,x);
    }
    
    public int length()
    {
        return length;
    }
    
    public int getNumSequences()
    {
        return numSequences;
    }
    
    public Sequence makeSequence(int index)
    {
        Sequence s=new Sequence(length);
        getSequence(index, s);
        return s;
    }
    
    public void getSequence(int index, Sequence s)
    {
        byte[] b=s.data();
        for (int i=0; i<length; i++)
        {
            b[i]=(byte)(index&0x3);
            index>>=2;
        }
    }
    
    public int indexOf(Sequence s)
    {
        if (s.length()!=length)
            throw new IllegalArgumentException("Length of sequence not right length; "+
                s.length()+" != real length, "+length);
        return indexOf(s,0);
    }
    
    public int indexOf(Sequence s, int start)
    {
        return findIndex(s.data(),start);
    }
    
    public float subsequenceValue(Sequence s, int start)//, int min)
    {
        int i=indexOf(s, start);
        //if (count[i]<min) return 0;
        return average(i);
    }
    
    public float subsequenceValue(Sequence s, int start, int min)
    {
        int i=indexOf(s, start);
        if (count[i]<min) return 0;
        return average(i);
    }
    
    public int subsequenceCount(Sequence s, int start)
    {
        return count[indexOf(s, start)];
    }
    
    public float average(int i)
    {
        int n=count[i];
        return n==0 ? 0 : value[i]/n;
    }
    
    public int count(int i)
    {
        return count[i];
    }
    
    private void include(byte[] data, int start, float x)
    {
        int index=findIndex(data,start);
        count[index]++;
        value[index]+=x;
    }
    
    private int findIndex(byte[] data, int start)
    {
        int index=0;
        for (int i=start+length-1; i>=start; i--)
        {
            index<<=2;
            index+=data[i];
        }
        return index;
    }
    
    private int sizeOf(int length)
    {
        int n;
        for (n=1; length>0; length--)
            n<<=2;
        return n;
    }
}