package x.feature;

import java.util.*;

import x.data.*;

public class FeatureArray
{
	private static String metaFile="c:\\data\\bio\\eisen-location.meta";
	private static String featureFile="c:\\data\\bio\\yeast\\chromosomal_feature_tab.txt";
	
	public MeasuredFeature[] measuredFeatures;
	public DataArray profiles;
	
	private int[] chromosomeMax=new int[1000];
	private int overallChromosomeMax;
	private int chromosomeCount;
	
	public FeatureArray() throws Exception
	{
		this(false);
	}
	
	public FeatureArray(boolean shuffle) throws Exception
	{
		profiles=new DataArray(metaFile);
		Feature[] features=FeatureReader.readFile(featureFile, "ORF");
		for (int i=0; i<features.length; i++)
		{
			Location l=features[i].getLocation();
			int chr=l.getChromosome();
			chromosomeCount=Math.max(chr+1, chromosomeCount);
			chromosomeMax[chr]=Math.max(chromosomeMax[chr], l.getStart());
			chromosomeMax[chr]=Math.max(chromosomeMax[chr], l.getStop());
			overallChromosomeMax=Math.max(overallChromosomeMax, chromosomeMax[chr]);
		}

		HashMap indexMap=new HashMap();
		for (int i=0; i<profiles.size(); i++)
		{
			String id=profiles.getRow(i).getID();
			indexMap.put(id, new Integer(i));
		}
		
		ArrayList featureList=new ArrayList();		
		for (int i=0; i<features.length; i++)
		{
			Integer index=(Integer)indexMap.get(features[i].getID());
			if (index!=null)
				featureList.add(
					new MeasuredFeature(features[i],profiles.getRow(index.intValue())));
		}
		int n=featureList.size();
		measuredFeatures=new MeasuredFeature[n];
		for (int i=0; i<n; i++)
		{
			measuredFeatures[i]=(MeasuredFeature)featureList.get(i);
		}
		if (shuffle)
			Stats.shuffle(measuredFeatures);
		else
			Arrays.sort(measuredFeatures);
		
		System.out.println("Matched ORFs: "+n);
		/*
		System.out.println("Normalizing all to unit sd.");
		int pn=profiles.size();
		for (int i=0; i<pn; i++)
			Stats.makeUnitSD(profiles.getRow(i).getNumbers());
		*/
		/*
			System.out.println("Normalizing all to unit range.");
			int pn=profiles.size();
			for (int i=0; i<pn; i++)
				Stats.makeUnitRange(profiles.getRow(i).getNumbers());
*/
	}
	
	public int getChromosomeMax(int i)
	{
		return chromosomeMax[i];
	}
	
	public int getOverallChromosomeMax()
	{
		return overallChromosomeMax;
	}
	
	public int getChromosomeCount()
	{
		return chromosomeCount;
	}
	
	public double[] findR()
	{
		int n=measuredFeatures.length;
		double[] r=new double[n-1];
		for (int i=1; i<n; i++)
		{
			MeasuredFeature m1=measuredFeatures[i];
			MeasuredFeature m0=measuredFeatures[i-1];
			r[i-1]=Stats.r(m1.row.getNumbers(),
						   m0.row.getNumbers());				
		}
		return r;
	}
}
