package x.feature;

import java.io.*;
import java.util.*;
import x.util.*;


/**
 * Designed to read the kind of feature
 * that comes in Stanford yeast tables.
 */
public class FeatureReader
{
	public static Feature[] readFile(String fileName) throws Exception
	{
		return readFile(fileName, null);
	}
	
	public static Feature[] readFile(String fileName, String typeFilter) throws Exception
	{
		ArrayList list=new ArrayList();
		
		File file=new File(fileName);
		BufferedReader in=new BufferedReader(new FileReader(file));
		int errors=0;
		try
		{
			String line=null;
			while (null != (line=in.readLine()))
			{
				try
				{
					FlexibleTokenizer t=new FlexibleTokenizer(line, "\t");
					String id=t.nextToken();
					t.nextToken(); // optional gene
					t.nextToken(); // optional alias
					String type=t.nextToken();
					
					if (typeFilter!=null && type.indexOf(typeFilter)==-1)
						continue;
					
					int chromosome=Integer.parseInt(t.nextToken());
					int start=Integer.parseInt(t.nextToken());
					int stop=Integer.parseInt(t.nextToken());
					char strand=t.nextToken().charAt(0);
					Location location=new Location(chromosome, start, stop, strand);
					Feature feature=new Feature(id, type, location);
					list.add(feature);
				}
				catch (Exception e)
				{
					++errors;
					if (errors<10)
						System.err.println("BAD LINE: "+line);
					else 
						if (errors==10)
							System.err.println("TOO MANY ERRORS, NOT PRINTING ANY MORE.");
				}
			}
			
		}
		finally
		{
			in.close();
		}
		System.err.println("Total bad lines="+errors);
		int n=list.size();
		Feature[] features=new Feature[n];
		return (Feature[])list.toArray(features);
	}
	
	public static void main(String[] args)
	{
		try
		{
			String file="c:\\data\\bio\\yeast\\chromosomal_feature_tab.txt";
			Feature[] f=readFile(file, "ORF");
			int n=f.length;
			int orf=0;
			for (int i=0; i<n; i++)
			{
				if (f[i].isType("ORF"))
					orf++;
			}
			System.out.println("ORFs: "+orf+" / "+n);
			System.out.println("\n\n==============\n\n");
			Arrays.sort(f);
			for (int i=0; i<Math.min(n, 50); i++)
				System.out.println(f[i]);
		}
		catch (Exception e)
		{
			e.printStackTrace(System.out);
		}
	}
}
