package x.feature;


public class Location implements Comparable
{
	private int start, stop, chromosome;
	private char strand;
	
	public Location(int chromosome, int start, int stop, char strand)
	{
		this.chromosome=chromosome;
		this.start=start;
		this.stop=stop;
		this.strand=strand;
	}
	
	public int max()
	{
		return Math.max(start, stop);
	}
	
	public int min()
	{
		return Math.min(start, stop);
	}
	
	public int distance(Location loc)
	{
		if (loc.chromosome!=chromosome)
			return -1;
		return Math.min(
			Math.abs(min()-loc.max()), Math.abs(loc.min()-max()));
	}
	
	public int compareTo(Object o)
	{
		Location loc=(Location)o;
		int c=chromosome-loc.chromosome;
		if (c!=0)
			return c;
		return start-((Location)o).start;
	}
	
	public String toString()
	{
		return "[chr="+chromosome+", "+start+"-"+stop+", strand="+strand+"]";
	}
	
	/**
	 * Returns the chromosome.
	 * @return int
	 */
	public int getChromosome()
	{
		return chromosome;
	}

	/**
	 * Returns the start.
	 * @return int
	 */
	public int getStart()
	{
		return start;
	}

	/**
	 * Returns the stop.
	 * @return int
	 */
	public int getStop()
	{
		return stop;
	}

	/**
	 * Returns the strand.
	 * @return char
	 */
	public char getStrand()
	{
		return strand;
	}

	/**
	 * Sets the chromosome.
	 * @param chromosome The chromosome to set
	 */
	public void setChromosome(int chromosome)
	{
		this.chromosome = chromosome;
	}

	/**
	 * Sets the start.
	 * @param start The start to set
	 */
	public void setStart(int start)
	{
		this.start = start;
	}

	/**
	 * Sets the stop.
	 * @param stop The stop to set
	 */
	public void setStop(int stop)
	{
		this.stop = stop;
	}

	/**
	 * Sets the strand.
	 * @param strand The strand to set
	 */
	public void setStrand(char strand)
	{
		this.strand = strand;
	}

}
