package x.feature;


public class Stats
{
	public static double r(double[] x, double[] y)
	{
		return cov(x,y)/Math.sqrt(var(x)*var(y));
	}
	public static float r(float[] x, float[] y)
	{
		return (float)(cov(x,y)/Math.sqrt(var(x)*var(y)));
	}
	
	public static double autoCorr(double[] x)
	{
		int n=x.length;
		double[] y0=new double[n-1];
		double[] y1=new double[n-1];
		System.arraycopy(x,0,y0,0,n-1);
		System.arraycopy(x,1,y1,0,n-1);
		return r(y0,y1);
	}
	
	public static double[] safe(double[] x)
	{
		int q=0;
		for (int i=0; i<x.length; i++)
			if (isOK(x[i]))
				q++;
		double[] y=new double[q];
		q=0;
		for (int i=0; i<x.length; i++)
			if (isOK(x[i]))
				y[q++]=x[i];
		return y;
	}
	
	private static boolean isOK(double x)
	{
		return x!=0 && !Double.isNaN(x) && !Double.isInfinite(x);
	}
	
	private static boolean isFinite(double x)
	{
		return !Double.isNaN(x) && !Double.isInfinite(x);
	}
	
	public static int numZero(double[] x)
	{
		int z=0;
		for (int i=0; i<x.length; i++)
			if (x[i]==0) z++;
		return z;
	}
	
	public static int numNaN(double[] x)
	{
		int z=0;
		for (int i=0; i<x.length; i++)
			if (Double.isNaN(x[i])) z++;
		return z;
		
	}
	
	public static double cov(double[] x, double[] y)
	{
		double sum=0;
		double mx=mean(x);
		double my=mean(y);
		for (int i=0; i<x.length; i++)
		{
			if (!Double.isNaN(x[i]) && !Double.isNaN(y[i]))
				sum+=(x[i]-mx)*(y[i]-my);
		}
		return sum/x.length;
		
	}
	
	public static float cov(float[] x, float[] y)
	{
		float sum=0;
		float mx=mean(x);
		float my=mean(y);
		for (int i=0; i<x.length; i++)
		{
			if (!Float.isNaN(x[i]) && !Float.isNaN(y[i]))
				sum+=(x[i]-mx)*(y[i]-my);
		}
		return (float)sum/x.length;
		
	}
	
	public static void makeUnitSD(float[] x)
	{
		float sd=sd(x);
		float m=mean(x);
		for (int i=0; i<x.length; i++)
		{
			x[i]=m+(x[i]-m)/sd;
		}
	}
	
	public static void makeUnitRange(double[] x)
	{
		double min=min(x);
		double max=max(x);
		for (int i=0; i<x.length; i++)
		{
			x[i]=(x[i]-min)/(max-min);
		}
	}
	
	public static void makeUnitRange(float[] x)
	{
		float min=min(x);
		float max=max(x);
		for (int i=0; i<x.length; i++)
		{
			x[i]=(float)(x[i]-min)/(max-min);
		}
	}
	
	public static float sd(float[] x)
	{
		return (float)Math.sqrt(var(x));
	}
	
	public static double sd(double[] x)
	{
		return Math.sqrt(var(x));
	}
	
	public static float var(float[] x)
	{
		double m=mean(x);
		return (float)(e2(x)-m*m);		
	}
	
	public static double var(double[] x)
	{
		double m=mean(x);
		return e2(x)-m*m;		
	}
	
	public static double mean(double[] x)
	{
		double sum=0;
		for (int i=0; i<x.length; i++)
		{
			if (!Double.isNaN(x[i]))
				sum+=x[i];
		}
		return sum/x.length;
	}
	
	public static float mean(float[] x)
	{
		float sum=0;
		for (int i=0; i<x.length; i++)
		{
			if (!Double.isNaN(x[i]))
				sum+=x[i];
		}
		return sum/x.length;
	}
	
	public static float e2(float[] x)
	{
		float sum=0;
		for (int i=0; i<x.length; i++)
		{
			if (!Double.isNaN(x[i]))
				sum+=x[i]*x[i];
		}
		return sum/x.length;
	}
	
	public static double e2(double[] x)
	{
		double sum=0;
		for (int i=0; i<x.length; i++)
		{
			if (!Double.isNaN(x[i]))
				sum+=x[i]*x[i];
		}
		return sum/x.length;
	}
	
	public static double min(double[] x)
	{
		double min=x[0];
		for (int i=1; i<x.length; i++)
		{
			min=Math.min(x[i], min);
		}
		return min;
	}
	
	public static double max(double[] x)
	{
		double max=x[0];
		for (int i=1; i<x.length; i++)
		{
			max=Math.max(x[i], max);
		}
		return max;
	}
	
	public static float min(float[] x)
	{
		double min=x[0];
		for (int i=1; i<x.length; i++)
		{
			min=Math.min(x[i], min);
		}
		return (float)min;
	}
	
	public static float max(float[] x)
	{
		double max=x[0];
		for (int i=1; i<x.length; i++)
		{
			max=Math.max(x[i], max);
		}
		return (float)max;
	}
	
	static void shuffle(Object[] o)
	{
		int n=o.length;
		for (int i=0; i<n; i++)
		{
			int j=(int)(n*Math.random());
			Object p=o[i];
			Object q=o[j];
			o[i]=q;
			o[j]=p;
		}
	}
	
	public static void main(String[] args)
	{
		double[] x={1,1,2,2,3,3,4,4,5,5};
		
		show(x);
		System.out.println("Auto cor: = "+autoCorr(x));
		System.out.println("max="+max(x));
		System.out.println("min="+max(x));
		makeUnitRange(x);
		show(x);
		/*
		double[] x={1,2,0,3, Double.NaN, 4};
		show(x);
		show(safe(x));
*/
		/*
		double[] x0={1,2,3,4,5};
		show(x0);
		System.out.println("mean: "+mean(x0));
		System.out.println("e2: "+e2(x0));
		System.out.println("var: "+var(x0));
		double[] y0={1,2,3,4,5};
		show(y0);
		System.out.println("r="+Stats.r(x0,y0));

		double[] y1={5,4,3,2,1};
		show(y1);
		System.out.println("r="+Stats.r(x0,y1));

		double[] y2={1,5,2,4,3};
		show(y2);
		System.out.println("cov="+Stats.cov(x0,y2));
		System.out.println("sd1="+Stats.sd(x0));
		System.out.println("sd2="+Stats.sd(y2));
		System.out.println("r="+Stats.r(x0,y2));

		//double[] y3={3,4,2,5,1};
		//show(y3);
		//System.out.println("r="+Stats.r(x0,y3));

		 */
	}
	
	static void show(double[] x)
	{
		System.out.print("( ");
		for (int i=0; i<x.length; i++)
			System.out.print(x[i]+" ");
		System.out.println(")");
	}
}
