package x.feature.early;

import x.data.*;
import x.feature.*;
import x.awt.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class LocationArrayMapMultiple extends Frame
{
	static String metaFile="c:\\data\\bio\\eisen-location.meta";
	static String featureFile="c:\\data\\bio\\yeast\\chromosomal_feature_tab.txt";
	
	Font labelFont=new Font("Helvetica", Font.PLAIN, 12);
	MeasuredFeature[] measuredFeatures;
	DataArray profiles;
/*
	Color lowColor=Color.black;
	Color medColor=Color.gray;
	Color highColor=Color.white;
*/
	
	Color lowColor=new Color(0,0,255);
	Color medColor=Color.white;
	Color highColor=new Color(255,0,0);
	
	//Color bgcolor=Color.white;
	//Color fgcolor=Color.black;
	private ColorScale valueScale=
		new GradientColorScale(lowColor,medColor,highColor, -.5,.5);
	Choice colChoice=new Choice();
	int colSelected=2;
	
	public static void main(String[] args)
	{
		try
		{
			DataArray profiles=new DataArray(metaFile);
			System.out.println("profiles: "+profiles.size());
			LocationArrayMapMultiple map=new LocationArrayMapMultiple(profiles);
			
			Feature[] features=FeatureReader.readFile(featureFile, "ORF");
			System.out.println("features: "+features.length);

			HashMap indexMap=new HashMap();
			for (int i=0; i<profiles.size(); i++)
			{
				String id=profiles.getRow(i).getID();
				indexMap.put(id, new Integer(i));
			}
			
			ArrayList featureList=new ArrayList();		
			for (int i=0; i<features.length; i++)
			{
				Integer index=(Integer)indexMap.get(features[i].getID());
				if (index!=null)
					featureList.add(
						new MeasuredFeature(features[i],profiles.getRow(index.intValue())));
			}
			int n=featureList.size();
			map.measuredFeatures=new MeasuredFeature[n];
			for (int i=0; i<n; i++)
			{
				map.measuredFeatures[i]=(MeasuredFeature)featureList.get(i);
			}
			
			System.out.println("Matched ORFs: "+n);
			Arrays.sort(map.measuredFeatures);
			map.profiles=profiles;
			map.setBounds(10,10,1000,700);
			map.setVisible(true);
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);
		}
	}
	
	static void shuffle(Object[] o)
	{
		int n=o.length;
		for (int i=0; i<n; i++)
		{
			int j=(int)(n*Math.random());
			Object p=o[i];
			Object q=o[j];
			o[i]=q;
			o[j]=p;
		}
	}
	
	public void paintGrid(Graphics g)
	{
		int dx=5, dy=30;
		int x=dx, y=dy;
		for (int i=0; i<measuredFeatures.length; i++)
		{
			MeasuredFeature m=measuredFeatures[i];
			draw(g, x, y, m.row.getNumbers());
			if (++y>getSize().height-5)
			{
				y=dy;
				x+=110;
			}
		}
	}
	
	public void paint(Graphics g)
	{
		int dx=5, dy=60;
		for (int i=0; i<measuredFeatures.length; i++)
		{
			MeasuredFeature m=measuredFeatures[i];
			drawSquare(g,i,m.row.getNumbers());
		}
		
	}
	

	
	void draw(Graphics g, int x, int y, float[] z)
	{
		for (int i=0; i<z.length; i++)
		{
			g.setColor(valueScale.color(z[i]));
			g.drawLine(x+i,y,x+i,y);
		}
	}
	
	void drawSquare(Graphics g, int j, float[] z)
	{
		int qx=(j%80);
		int qy=(j/80);
		for (int i=0; i<z.length; i++)
		{
			g.setColor(valueScale.color(z[i]));
			int x=90*(i%11)+15;
			int y=95*(i/11)+50;
			g.drawLine(x+qx,y+qy,x+qx,y+qy);
			if (j==0)
			{
				g.setColor(Color.black);
				String label=profiles.getFields()[i+2].getName();
				int l=label.length();
				if (l>15)
					label=label.substring(0,8)+"..."+label.substring(l-6,l);
				g.drawString(label,x,y-2);
			}
		}
	}
	
	LocationArrayMapMultiple(DataArray data)
	{
		super("LocationArrayMap");
		setBackground(new Color(230,230,230));
		setFont(new Font("Helvetica", Font.PLAIN, 9));
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		});
	}
}
