package x.feature.early;

import x.data.*;
import x.feature.*;
import x.awt.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class LocationArrayScatter extends Frame
{
	static String metaFile="c:\\data\\bio\\eisen-location.meta";
	static String featureFile="c:\\data\\bio\\yeast\\chromosomal_feature_tab.txt";
	
	Font labelFont=new Font("Helvetica", Font.PLAIN, 12);
	MeasuredFeature[] measuredFeatures;
	DataArray profiles;
	/*
	Color lowColor=new Color(0,255,0);
	Color medColor=Color.black;
	Color highColor=new Color(255,255,255);
	*/
	Color lowColor=new Color(0,0,255);
	Color medColor=Color.white;
	Color highColor=new Color(255,0,0);
	Color bgcolor=Color.white;
	Color fgcolor=Color.black;
	private ColorScale valueScale=
		new GradientColorScale(lowColor,medColor,highColor, -.5,.5);
	Choice colChoice=new Choice();
	int colSelected=2;

	public static void main(String[] args)
	{
		try
		{
			DataArray profiles=new DataArray(metaFile);
			System.out.println("profiles: "+profiles.size());
			LocationArrayScatter map=new LocationArrayScatter(profiles);
			
			Feature[] features=FeatureReader.readFile(featureFile, "ORF");
			System.out.println("features: "+features.length);
			//Arrays.sort(features);
			//shuffle(features);
			HashMap indexMap=new HashMap();
			for (int i=0; i<profiles.size(); i++)
			{
				String id=profiles.getRow(i).getID();
				indexMap.put(id, new Integer(i));
			}
			
			ArrayList featureList=new ArrayList();		
			for (int i=0; i<features.length; i++)
			{
				Integer index=(Integer)indexMap.get(features[i].getID());
				if (index!=null)
					featureList.add(
						new MeasuredFeature(features[i],profiles.getRow(index.intValue())));
			}
			int n=featureList.size();
			map.measuredFeatures=new MeasuredFeature[n];
			for (int i=0; i<n; i++)
			{
				map.measuredFeatures[i]=(MeasuredFeature)featureList.get(i);
			}
			
			System.out.println("Matched ORFs: "+n);
			Arrays.sort(map.measuredFeatures);
			map.profiles=profiles;
			map.setBounds(10,10,1000,700);
			map.setVisible(true);
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);
		}
	}
	
	static void shuffle(Object[] o)
	{
		int n=o.length;
		for (int i=0; i<n; i++)
		{
			int j=(int)(n*Math.random());
			Object p=o[i];
			Object q=o[j];
			o[i]=q;
			o[j]=p;
		}
	}
	
	public void paintGrid(Graphics g)
	{
		int dx=5, dy=30;
		int x=dx, y=dy;
		for (int i=0; i<measuredFeatures.length; i++)
		{
			MeasuredFeature m=measuredFeatures[i];
			draw(g, x, y, m.row.getNumbers());
			if (++y>getSize().height-5)
			{
				y=dy;
				x+=110;
			}
		}
	}
	
	public void paint(Graphics g)
	{
		//int dx=5, dy=60;
		//int p=profiles.getRow(0).getNumbers().length;
		g.setFont(labelFont);
		drawColumn(g,200,100,colSelected, 6);
		/*
		for (int i=0; i<p; i++)
		{
			int x=180*(i%5)+dx;
			int y=180*(i/5)+dy;
			drawColumn(g,x,y,i,2);
		}
		*/
	}
	
	void drawColumn(Graphics g, int x, int y, int col, int s)
	{
		g.setColor(fgcolor);
		g.drawString(profiles.getFields()[col+2].getName(), x, y);
		int c=0;		
		int dy=0;
		
		int cx=x, cy=y;
		int wx=x+60*s+50;
		int wbase=wx;
		int wy=y;
		for (int i=1; i<measuredFeatures.length; i++)
		{
			MeasuredFeature m1=measuredFeatures[i-1];
			MeasuredFeature m2=measuredFeatures[i];
			int chr1=m1.feature.getLocation().getChromosome();
			int chr2=m2.feature.getLocation().getChromosome();
			if (chr1!=chr2) continue;
			
			double v1=m1.row.getNumbers()[col];
			double v2=m2.row.getNumbers()[col];
			g.setColor(Color.black);
			int a=x+200+(int)(50*v1);
			int b=y+200-(int)(50*v2);
			g.drawLine(a,b,a,b);
			//g.fillRect(a-1,b-1,3,3);
			/*
			boolean isC=m.feature.getLocation().getStrand()=='C';
			int a=(isC ? cx : wx);
			int b=(isC ? cy : wy);
			g.fillRect(a,b,s,s);
			if (isC)
			{
				cx+=s;
				if (cx>x+60*s)
				{
					cx=x;
					cy+=s;
				}
			}
			else
			{
				wx+=s;
				if (wx>wbase+60*s)
				{
					wx=wbase;
					wy+=s;
				}
			}
			*/
		}
	}
	
	void draw(Graphics g, int x, int y, float[] z)
	{
		for (int i=0; i<z.length; i++)
		{
			g.setColor(valueScale.color(z[i]));
			g.drawLine(x+i,y,x+i,y);
		}
	}
	
	void drawSquare(Graphics g, int j, float[] z)
	{
		int qx=(j%80);
		int qy=(j/80);
		for (int i=0; i<z.length; i++)
		{
			g.setColor(valueScale.color(z[i]));
			int x=90*(i%11)+15;
			int y=90*(i/11)+30;
			g.drawLine(x+qx,y+qy,x+qx,y+qy);
		}
	}
	
	LocationArrayScatter(DataArray data)
	{
		super("LocationArrayMap");
		setBackground(new Color(230,230,230));
		//setDoubleBuffered(true);
		setLayout(null);	
		for (int i=2; i<data.getFields().length; i++)
			colChoice.addItem(data.getFields()[i].getName());
		add(colChoice);
		colChoice.setBounds(5,30,150,25);	
		colChoice.addItemListener(new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				colSelected=colChoice.getSelectedIndex();
				repaint();
			}
		});
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		});
	}
}
