package x.math;

/**
 *
 * Class holding static methods to perform geometric
 * calculations in Euclidean space.
 *
 */
public class Geometry
{
    public static void unit(float[] x)
    {
        float d=len(x);
        if (d==0) throw new IllegalArgumentException("Can't make unit vector from 0");
        mul(x, 1/d);
    }
    
    public static void mul(float[] x, float a)
    {
        for (int i=x.length-1; i>=0; i--)
            x[i]*=a;
    }
    
    public static final float mean(float[] x)
    {
    	return sum(x)/x.length;
    }
    
    public static final float corr(float[] x, float[] y)
    {
    	float xbar=mean(x);
    	float ybar=mean(y);
    	int n=x.length;
    	float c=0,sx=0,sy=0;
    	for (int i=n-1; i>=0; i--)
    	{
    		float x1=x[i]-xbar;
    		float y1=y[i]-ybar;
    		c+=x1*y1;
    		sx+=x1*x1;
    		sy+=y1*y1;
    	}
    	return c/(float)Math.sqrt(sx*sy);
    }
    
    public static final float cov(float[] x, float[] y)
    {
        return dot(x,y)/(len(x)*len(y));
    }
    
    public static final float dist(float[] x, float[] y)
    {
        return (float)Math.sqrt(d2(x,y));
    }
    
    public static final float d2(float[] x, float[] y)
    {
        float s=0;
        for (int i=0; i<x.length; i++)
        {
            s+=(x[i]-y[i])*(x[i]-y[i]);
        }
        return s;
    }
    
    public static final float len(float[] x)
    {
        float s=0;
        for (int i=0; i<x.length; i++)
            s+=x[i]*x[i];
        return (float)Math.sqrt(s);
    }
    
    public static final float dot(float[] x, float[] y)
    {
        float s=0;
        for (int i=0; i<x.length; i++)
            s+=x[i]*y[i];
        return s;
    }
    
    public static final float sum(float[] x)
    {
        float s=0;
        for (int i=0; i<x.length; i++)
            s+=x[i];
        return s;
    }
    
}