package x.path;

import java.awt.*;

public class Grid
{
    private Rectangle screenBounds, gridBounds;
    
    public Grid(Dimension screenSize, Dimension gridSize)
    {
        this(new Rectangle(0,0,screenSize.width,screenSize.height),
             new Rectangle(0,0,gridSize.width,gridSize.height));
    }
    
    public Grid(Rectangle screenBounds, Rectangle gridBounds)
    {
        setScreenBounds(screenBounds);
        setGridBounds(gridBounds);
    }
    
    public void setScreenBounds(Rectangle screenBounds)
    {
        this.screenBounds=screenBounds;
    }
    
    public void setGridBounds(Rectangle gridBounds)
    {
        this.gridBounds=gridBounds;
    }
    
    public Rectangle getScreenBounds()
    {
        return screenBounds;
    }
    
    public Rectangle getGridBounds()
    {
        return gridBounds;
    }
    
    private int scale(int source, int sourceLow, int sourceHigh, int targetLow, int targetHigh)
    {
        return targetLow+(int)((source-sourceLow)*(targetHigh-targetLow)/(double)(sourceHigh-sourceLow));
    }
    
    public int getGridX(int screenX)
    {
        return scale(screenX, screenBounds.x, screenBounds.width, gridBounds.x, gridBounds.width);
    }
    
    public int getGridY(int screenY)
    {
        return scale(screenY, screenBounds.y, screenBounds.height, gridBounds.y, gridBounds.height);
    }
    
    public int getScreenX(int gridX)
    {
        return scale(gridX, gridBounds.x, gridBounds.width, screenBounds.x, screenBounds.width);
    }
    
    public int getScreenY(int gridY)
    {
        return scale(gridY, gridBounds.y, gridBounds.height, screenBounds.y, screenBounds.height);
    }
    
    public void getScreenRect(Rectangle screenRect, Point gridPoint)
    {
        int x=getScreenX(gridPoint.x);
        int w=Math.max(1,getScreenX(gridPoint.x+1)-x);
        int y=getScreenY(gridPoint.y);
        int h=Math.max(1,getScreenY(gridPoint.y+1)-y);
        screenRect.setBounds(x,y,w,h);
    }
    
    public void getScreenRect(Rectangle screenRect, int gx, int gy)
    {
        int x=getScreenX(gx);
        int w=Math.max(1,getScreenX(gx+1)-x);
        int y=getScreenY(gy);
        int h=Math.max(1,getScreenY(gy+1)-y);
        screenRect.setBounds(x,y,w,h);
    }
    
    public void getScreenCenter(Point screenCenter, int gridX, int gridY)
    {
        int x1=getScreenX(gridX);
        int x2=getScreenX(gridX+1);
        int y1=getScreenY(gridY);
        int y2=getScreenY(gridY+1);
        screenCenter.x=(x1+x2)/2;
        screenCenter.y=(y1+y2)/2;
    }
    
    public void getScreenCenter(Point screenCenter, Point gridPoint)
    {
        getScreenCenter(screenCenter, gridPoint.x, gridPoint.y);
    }
    
    public void getGridCoordinates(Point gridPoint, Point screenPoint)
    {
        gridPoint.x=getGridX(screenPoint.x);
        gridPoint.y=getGridY(screenPoint.y);
    }
}
    
