package x.path;

import java.awt.*;

public class HilbertPath implements PathLayout
{
    private int width;
    private int height;
    private int length;
    private Point[] path;
    private int[][] order;
    
    private final Point[] dir={
           new Point(0,1), new Point(1,0), new Point(0,-1), new Point(-1,0)
    };
    
    public HilbertPath(int length, int w, int h)
    {
        setMaxBounds(length,w,h);
    }
    
    public int order(int x, int y)
    {
        if (x>=0 && y>=0 && x<width && y<height)
            return order[x][y];
        return -1;
    }
    
    public boolean contains(int x, int y)
    {
        int n=order(x,y);
        return n>=0 && n<length;
    }
    
    public int x(int n)
    {
        return path[n].x;
    }
    
    public int y(int n)
    {
        return path[n].y;
    }
    
    public Point getPoint(int n)
    {
        return path[n];
    }
    
    public Dimension getSize()
    {
        return new Dimension(width, height);
    }
    
    public int getLength()
    {
        return length;
    }
    
    public void setMaxBounds(int length, int width, int height)
    {
        this.length=length;
        this.width=width;
        this.height=height;
        makePath();
    }
    
    
    private void makePath()
    {
        int n=1;
        int side=1;
        while (n<length)
        {
            n*=4;
            side*=2;
        }
        
        path=new Point[n];
        order=new int[side][side];
        for (int x=0; x<side; x++)
            for (int y=0; y<side; y++)
                order[x][y]=-1;
        
        int i;
        Point[] dpath=new Point[n-1];
        for (i=0; i<dpath.length; i++)
            dpath[i]=dir[f(n,i)];
        
        // peano curve
        int px=0,py=0;
        order[px][py]=0;
        int actualWidth=0;
        int actualHeight=0;
        for (i=0; i<n; i++)
        {
            path[i]=new Point(px,py);
            actualWidth=Math.max(actualWidth, px);
            actualHeight=Math.max(actualHeight, py);
            
            order[px][py]=i;
            if (i==dpath.length) break;
            px+=dpath[i].x;
            py+=dpath[i].y;
            
        }
        
        width=actualWidth+1;
        height=actualHeight+1;
    }    
    
    
    private int f(int n, int p)
    {
        if (n<=1) return p;
        int power=1<<(2*n-2);
        int q=p%power;
        int d=p/power;
        if (q==power-1) return d;
        int x=f(n-1,q);
        if (d==0) return (3*x+1)%4;
        if (d==3) return (3*x+3)%4;
        return x;
    }
    
}
