package x.util;

/**
 *
 * This class is similar to StringTokenizer, except that
 * it doesn't coalesce adjacent tokens. For example,
 * if you use StringTokenizer to tokenize "A,B,,X" with a comma
 * as the token, then you will get three tokens "A", "B", and "X".
 * EmptyFieldTokenizer, on the other hand, will return four tokens,
 * "A", "B", the empty string "", and "X". This is often useful in
 * parsing machine-generated lists of Strings.
 *
 */
public class FlexibleTokenizer
{
    private String base, delimiters;
    private int index;
    
    public FlexibleTokenizer(String base, String delimiters)
    {
        this.base=base;
        this.delimiters=delimiters;
        index=-1;
    }
    
    public int countTokens()
    {
        int n=1;
        for (int i=base.length()-1; i>=0; i--)
            if (isDelimiter(i))
                n++;
        return n;
    }
    
    public String nextToken()
    {
        
        int i=index;
        int length=base.length();
        if (i>=length)
            throw new java.util.NoSuchElementException("No more tokens.");
            
        do {index++;}
        while (index<length && !isDelimiter(index));
        
        return base.substring(i+1, index);
    }
    
    public boolean hasMoreTokens()
    {
        return index<base.length();
    }
    
    private boolean isDelimiter(int i)
    {
        return delimiters.indexOf(base.charAt(i))!=-1;
    }
}