package x.viz.biplot;

import java.awt.*;
import java.awt.image.*;


public class DensityBiplotRenderer
{
    private PlotPoint[] points;
    private int[] density1, density2;
    private int width, height;
    private byte[] pixels;
    private int[] overlay1, overlay2;
    
    private static final byte POINT_COLOR=-1;
    
    private IndexColorModel model;
    
    
    public DensityBiplotRenderer(int width, int height, int[] density1, int[] density2, PlotPoint[] points)
    {
        this.width=width;
        this.height=height;
        this.points=points;

        this.density1=density1;
        this.density2=density2;
                
        pixels=new byte[width*height];
        overlay1=new int[width];
        overlay2=new int[height];

        // make palette for indexed color model:
        byte[] r=new byte[256];
        byte[] g=new byte[256];
        byte[] b=new byte[256];
        for (int i=0; i<256; i++)
        {
            r[i]=(byte)(i/2);
            g[i]=(byte)(i/2);
            b[i]=(byte)(60+(3*i)/4);
        }
        
        r[0]=0;
        g[0]=0;
        b[0]=50;
        
        r[255]=-1;
        g[255]=-1;
        b[255]=-1;
        model=new IndexColorModel(8,256,r,g,b);
    }
    
    public Image makeImage(int start1, int end1, int start2, int end2)
    {
        fill(start1, end1, start2, end2);
        Image image= Toolkit.getDefaultToolkit().createImage(
            new MemoryImageSource(width, height, model, pixels, 0, width));

        return image;
    }
    
    void fill(int start1, int end1, int start2, int end2)
    {

        makeOverlay(overlay1, density1, start1, end1);
        
        //for (int i=0; i<50; i++)
        //    System.out.println(overlay1[i]);
        
        makeOverlay(overlay2, density2, start2, end2);
        combineOverlays();
        
        int n=points.length;
        int m1=end1-start1;
        int m2=end2-start2;
        
        for (int i=0; i<n; i++)
        {
            int x=points[i].x;
            int y=points[i].y;
            if (x<start1 || x>end1 || y<start2 || y>end2) continue;
            
            int a0=(width*(x-start1))/m1;
            int b0=(height*(y-start2))/m2;
            int a1=Math.max(a0,(width*(x+1-start1))/m1-1);
            int b1=Math.max(b0,(height*(y+1-start2))/m2-1);
            
            for (int a=a0; a<=a1; a++)
            for (int b=b0; b<=b1; b++)
            {
                int q=width*b+a;
                if (q>=0 && q<pixels.length)
                    pixels[q]=POINT_COLOR;
            }
        }
        
        
    }
    
    void combineOverlays()
    {
        int m=0;
        for (int y=0; y<height; y++)
        {
            if (overlay2[y]!=0)
                for (int x=0; x<width; x++)
                {
                    if (overlay1[x]!=0)
                    {
                        byte value= (byte)(1+overlay1[x]+overlay2[y]);
                        pixels[m]=value;
                    }
                    else
                        pixels[m]=0;
                    m++;
                }
            else
                for (int x=0; x<width; x++)
                {
                    pixels[m]=0;
                    m++;
                }
        }
    }
    
    void makeOverlay(int[] overlay, int[] density, int start, int end)
    {
        int n=end-start;
        int oldX=start;
        
        //System.out.println("start, end: "+start+", "+end);
        
        if (overlay.length<=0)
            throw new IllegalArgumentException("overlay length was: "+overlay.length);

        for (int i=0; i<overlay.length; i++)
        {
            int x=start+(n*i)/(overlay.length-1);
            int value=0;
            int nonzero=0;
            for (int j=oldX; j<=x; j++)
            {
                value+=density[j];
                if (density[j]!=0)
                    nonzero++;
            }
            
            //if (x-oldX+1==0)
            //    System.out.println("x="+x+", overlay.length="+overlay.length+
            //     ", i="+i+", start="+start+", end="+end);
            value/=(x-oldX+1);
            //if (nonzero>1) value/=nonzero;
            
            overlay[i]=value;

            oldX=x;
        }
    }

    void clear()
    {
        for (int i=pixels.length-1; i>=0; i--)
            pixels[i]=0;
    }

    public void time()
    {
        int n=100;
        int i;

        long start=System.currentTimeMillis();
        for (i=0; i<n; i++)
            makeImage(0,0,width,height);
        long fillTime=(System.currentTimeMillis()-start)/n;
        System.out.println("image time: "+fillTime);
    }
    
}