package x.viz.biplot;

import x.awt.*;
import java.awt.*;
import java.awt.event.*;

public class DensityBiplotViewer extends BufferPanel implements Runnable
{
    DensityBiplotRenderer renderer;
    int x1,y1,x2,y2;
    int width, height;
    int length1, length2;
    private Thread animation;
    private boolean mouseIsDown;
    private boolean isMetaDown;
    private Point mouse=new Point(0,0);
    private int[] density1, density2;
    private PlotPoint[] points;
    
    
    //public DensityBiplotViewer(double[] xDensity, double[] yDensity, PlotPoint[] points)
    public DensityBiplotViewer(BiplotModel model)
    {
        double[] xDensity=model.xDensity;
        double[] yDensity=model.yDensity;
        PlotPoint[] points=model.points;
        
        
        length1=xDensity.length;
        length2=yDensity.length;
        x1=0;
        x2=length1-1;
        y1=0;
        y2=length2-1;
        
        this.points=points;
        density1=makeInt(xDensity);
        density2=makeInt(yDensity);
        
        addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent m)
            {
                handle(m,true);
            }
            public void mouseReleased(MouseEvent m)
            {
                handle(m,false);
            }
        });
        
        addMouseMotionListener(new MouseMotionAdapter()
        {
            public void mouseDragged(MouseEvent m)
            {
                handle(m, true);
            }
        });
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        renderer=new DensityBiplotRenderer(w, h, density1, density2, points);
        width=w;
        height=h;
        super.setBounds(x,y,w,h);
    }
    
    int[] makeInt(double[] d)
    {
        int n=d.length;
        int[] m=new int[n];
        
        for (int i=0; i<n; i++)
            m[i]=Math.min(126, Math.max(0, (int)(127*d[i])));
        
        return m;
    }
    
    void handle(MouseEvent m, boolean mouseIsDown)
    {
        this.mouseIsDown=mouseIsDown;
        isMetaDown= 0 != (m.getModifiers() & m.BUTTON3_MASK);
        mouse.x=m.getX();
        mouse.y=m.getY();
    }
    
    public synchronized void start()
    {
        animation=new Thread(this);
        animation.start();
    }
    
    public synchronized void stop()
    {
        animation=null;
    }
    
    public void run()
    {
        //int n=0;
        for (;;)
        {
            //n++;
            //if (n%5==0) System.gc();
            if (mouseIsDown && renderer!=null && width>0 && height>0)
            synchronized (this)
            {
                double u=.5;
                double v=.5;
                double factor=.05* (isMetaDown ? 1 : -1);
                x1-=(int)(factor*u*(x2-x1));
                x2+=(int)(factor*(1-u)*(x2-x1));
                y1-=(int)(factor*v*(y2-y1));
                y2+=(int)(factor*(1-v)*(y2-y1));
                
                double mx=mouse.x/(double)width-.5;
                double my=mouse.y/(double)height-.5;
                int dx=(int)(.1*mx*(x2-x1));
                dx=Math.max(dx, -x1);
                dx=Math.min(dx, length1-1-x2);
                //if (x1+dx && x2<length1-dx)
                //{
                    x1+=dx;
                    x2+=dx;
                //}
                int dy=(int)(.1*my*(y2-y1));
                dy=Math.max(dy, -y1);
                dy=Math.min(dy, length2-1-y2);
                //if (y1>dy && y1<length2-dy)
                //{
                    y1+=dy;
                    y2+=dy;
                //}
                if (x1<0) x1=0;
                if (x2>=length1) x2=length1-1;
                if (y1<0) y1=0;
                if (y2>=length2) y2=length2-1;
                
                redraw();
            }
            
            try {Thread.sleep(50);} catch (Exception e) {}
        }
    }
    
    protected synchronized void drawOffscreen(Graphics g)
    {
        if (renderer!=null)
            g.drawImage(renderer.makeImage(x1,x2,y1,y2),0,0,null);
    }
}
