package x.viz.biplot;

import x.util.*;
import java.util.*;

public class Gap implements Ordering
{
    public int position;
    public int thickness;
    
    public Gap()
    {
    }
    
    public Gap(int position, int thickness)
    {
        this.position=position;
        this.thickness=thickness;
    }
    
    public String toString()
    {
        return "Gap: p="+position+", t="+thickness;
    }
    
    public int order(Object x, Object y)
    {
        Gap g1=(Gap)x;
        Gap g2=(Gap)y;
        return g2.position-g1.position;
    }
    
    public static BiplotModel addGaps(BiplotModel model, Vector xGap, Vector yGap)
    {
        return addGaps(model, gapArray(xGap), gapArray(yGap));
    }
    
    static Gap[] gapArray(Vector v)
    {
        int n=v.size();
        Gap[] g=new Gap[n];
        v.copyInto(g);
        return g;
    }
    
    // The gap arrays will be sorted.
    // Items in the plotpoint array will be modified.
    public static BiplotModel addGaps(BiplotModel model, Gap[] xGap, Gap[] yGap)
    {
        Gap ordering=new Gap();
        Sort10.sort(ordering, xGap);
        Sort10.sort(ordering, yGap);
        
        double[] xDensity=model.xDensity;
        int xn=xDensity.length;
        double[] yDensity=model.yDensity;
        int yn=yDensity.length;
        PlotPoint[] points=model.points;
        
        int xadd=sum(xGap);
        int yadd=sum(yGap);
        
        double[] xNew=new double[xn+xadd];
        double[] yNew=new double[yn+yadd];
        
        int[] xmap=spread(xDensity, xNew, xGap);
        int[] ymap=spread(yDensity, yNew, yGap);
        
        for (int i=0; i<points.length; i++)
        {
            points[i].x=xmap[points[i].x];
            points[i].y=ymap[points[i].y];
        }
        
        return new BiplotModel(xNew, yNew, points);
    }
    
    // Add gaps to sequence in "s1", putting the
    // result in "s2", and return a coordinate map from old to new.
    public static int[] spread(double[] s1, double[] s2, Gap[] gap)
    {
        int n1=0;
        int n2=0;
        int ng=0;
        
        int[] map=new int[s1.length];
        for (n1=0; n1<s1.length; n1++)
        {
            map[n1]=n2;
            s2[n2++]=s1[n1];
            while (ng<gap.length && gap[ng].position==n1)
            {
                for (int i=0; i<gap[ng].thickness; i++)
                    s2[n2++]=0;
                ng++;
            }
        }
        
        return map;
    }
    
    public static int sum(Gap[] gap)
    {
        int total=0;
        for (int i=0; i<gap.length; i++)
            total+=gap[i].thickness;
        return total;
    }
}