package x.viz.seq;

import x.data.*;
import x.data.seq.*;
import x.awt.*;
import x.util.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;
import java.util.*;
import java.text.*;

public class MotifViewer extends Frame implements Ordering
{ 
    DataArray data;
    SequenceValue v;
    double min=-.05, max=.05;
    ColorScale color=new GradientColorScale(Color.green, Color.darkGray, 
        Color.red, min, max);
    ColorScale highlightScale=new GradientColorScale(Color.gray, Color.gray, Color.white, min, max);

    AnnotatedSequence[] seq;
    SequenceListViewer overview;
    SequenceDetailViewer details;
    int column=0;
    int length=6;
    final static int MOTIF_VALUE=0, MOTIF_TARGET=1;
    int highlight=MOTIF_VALUE;
    ControlPanel controls;

    
    
    public static void main(String[] args)
    {
        if (args.length!=2)
        {
            System.err.println("usage: MotifViewer metafile seqfile");
			System.err.println("using defaults instead...");
			args=new String[2];
			args[0]="eisen.meta";
			args[1]="utr5_sc_500.fasta";
        }
        
        
        try
        {
            MotifViewer c=new MotifViewer();
            c.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {System.exit(0);}
            });
            c.setBounds(50,10,1000,1000);
            c.init(args[0], args[1]);
            c.setVisible(true);
        }
        catch (Exception e)
        {
            e.printStackTrace(System.err);
        }
    }
    int start=500;
    
    public MotifViewer()
    {
        super("Motif Viewer");
    }
    
    
    public void init(String metaFile, String seqFile) throws Exception
    {
        int i;
        
        
        // Read data.
        data=new DataArray(metaFile);
        
        // Read basic sequences.
        Sequence[] unordered=FASTA.readFromFile(seqFile);
        for (i=0; i<unordered.length; i++)
            unordered[i]=unordered[i].first(500);
        
        // Translate to marked sequences.
        AnnotatedSequence[] u=UTR.translate(unordered);
        
        
        // Make AnnotatedSequence table.
        Hashtable seqTable=new Hashtable();
        for (i=0; i<u.length; i++)
            seqTable.put(u[i].getID(), u[i]);
        
        // Make averages.
        int goodcount=0;
        seq=new AnnotatedSequence[data.size()];
        
        // Correlate with data.
        for (i=0; i<data.size(); i++)
        {
            DataRow d=data.getRow(i);
            StringTokenizer t=new StringTokenizer(d.getText()[1]);
            String id=t.nextToken();
            AnnotatedSequence m=(AnnotatedSequence)seqTable.get(id);
            if (m!=null)
            {
                m.setDataIndex(i);
                seq[goodcount]=m;
                goodcount++;
            }
        }
        
        AnnotatedSequence[] temp=seq;
        seq=new AnnotatedSequence[goodcount];
        System.arraycopy(temp,0,seq,0,goodcount);
        System.out.println("processed AnnotatedSequences: good="+goodcount+" out of "+u.length);
         
        // Make a sequence model.
        SequenceListModel model=makeModel(length);
        
        // Display it in a viewer.
        overview=new SequenceListViewer(model);
        
        // Make a details viewer.
        details=new SequenceDetailViewer(model, data, seq);
        
        setLayout(new BorderLayout());
        
        Panel vp=new Panel();
        add(vp, BorderLayout.CENTER);
        
        vp.setLayout(new GridLayout(1,2));
        vp.add(overview);
        overview.redraw();
        vp.add(details);
        details.setBackground(Color.black);
        details.redraw();
        
        overview.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                Rectangle r=overview.getFocusRegion();
                if (e.getActionCommand()==overview.VIEW_CHANGE)
                    details.setStart(r.x, r.y);
            }
        });
        
        controls=new ControlPanel();
        add(controls, BorderLayout.NORTH);
        
    }
    
    class ControlPanel extends Panel implements ItemListener, ActionListener
    {
        Choice valueChoice;
        Hashtable fieldTable=new Hashtable();
        TextField targetField;
        Button targetButton;
        
        public ControlPanel()
        {
            setBackground(Color.darkGray);
            setLayout(null);
            // Add value choice.
            valueChoice=new Choice();
            DataField[] field=data.getFields();
            int count=0;
            for (int i=0; i<field.length; i++)
            {
                int type=field[i].getType();
                String name=field[i].getName();
                if (type==MetaData.NUMBER && name.trim().length()>0)
                {
                    count++;
                    name=count+". "+name;
                    fieldTable.put(name, field[i]);
                    valueChoice.addItem(name);
                }
            }
            add(valueChoice);
            valueChoice.addItemListener(this);
            valueChoice.setBounds(10,5,215,20);
            
            Label targetLabel=new Label("Search for: ");
            add(targetLabel);
            targetLabel.setBackground(getBackground());
            targetLabel.setForeground(Color.white);
            targetLabel.setBounds(250,5,60,20);
            
            targetField=new TextField();
            add(targetField);
            targetField.setBounds(320,5,90,20);
            targetField.addActionListener(this);
            
            setColumn();
        }
        
        public void actionPerformed(ActionEvent e)
        {
            if (e.getSource()==targetField)
            {
                String s=targetField.getText();
                if (s.length()>0)
                    setTarget(s);
            }
        }
        
        String getTarget()
        {
            return targetField.getText();
        }
    
        public void itemStateChanged(ItemEvent e)
        {
            if (e.getSource()==valueChoice)
            {
                setColumn();
            }
        }
        
        void setColumn()
        {
            DataField d=(DataField)fieldTable.get(valueChoice.getSelectedItem());
            MotifViewer.this.setColumn(d.getColumnNum());
        }
        
        
        public Dimension getPreferredSize()
        {
            return new Dimension(1000,40);
        }
    }
    
    void setTarget(String target)
    {
        highlight=MOTIF_TARGET;
        makeNewModel();
    }
    
    void setColumn(int column)
    {
        highlight=MOTIF_VALUE;
        this.column=column;
        makeNewModel();
    }
    
    void makeNewModel()
    {
        SequenceListModel model=makeModel(length);
        overview.setModel(model);
        details.setModel(model);
    }
    
    public int order(Object x, Object y)
    {
        AnnotatedSequence a1=(AnnotatedSequence)x;
        AnnotatedSequence a2=(AnnotatedSequence)y;
        double a=data(a1)-data(a2);
        if (a<0) return -1;
        if (a>0) return 1;
        return 0;
    }
    
    float data(AnnotatedSequence s)
    {
        int d=s.getDataIndex();
        return data.getRow(d).getNumbers()[column];
    }
    
    SequenceListModel makeModel(int length)
    {
        // Find motif values.
        findValues(seq, data, length);
        
        int n=seq.length;
        Sequence[] value=new Sequence[n];
        double[] summaryValue=new double[n];

        Sequence target=null;
        int targetLength=0;
        if (highlight==MOTIF_TARGET)
        {
            target=new Sequence(controls.getTarget());
            targetLength=target.length();
        }
        for (int i=0; i<n; i++)
        {
            int d=seq[i].getDataIndex();
            summaryValue[i]=data.getRow(d).getNumbers()[column];
            Sequence s=seq[i].getSequence();
            int m=s.length()-length;
            value[i]=new Sequence(m);
            int j;
            for (j=0; j<m; j++)
            {
                if (highlight==MOTIF_VALUE)
                {
					int sv=(int)(-300*v.subsequenceValue(s,j,10));
					//int sv=-(int)(300*v.subsequenceValue(s,j,10));
                    if (sv<-127) sv=-127;
                    if (sv>127) sv=127;
                    value[i].setData(j, (byte)sv);
                }
                else
                {
                     boolean match=seq[i].getSequence().match(target,j,0,targetLength);
                     value[i].setData(j, (byte)(match ? 127 : 0));
                }
            }
            
            Sequence maxValue=new Sequence(m);
            int maxLen=(highlight==MOTIF_TARGET ? targetLength :length);
            for (j=0; j<m; j++)
            {
                int max=value[i].data(j);
                for (int k=-maxLen+1; k<0; k++)
                {
                    if (j+k<0) continue;
                    int q=(int)value[i].data(j+k);
                    if (Math.abs(q)>Math.abs(max))
                        max=q;
                }
                maxValue.setData(j,(byte)max);
            }
            value[i]=maxValue;
        }

        ByteColorScale d=new ByteColorScale(highlight==MOTIF_VALUE ? color :
            highlightScale, min, max);
        //ColorScale d=new DNAColorScale();
        return new SequenceListModel(value, summaryValue, d);
    }
    
    void findValues(AnnotatedSequence[] seq, DataArray data, int length)
    {
        v=new SequenceValue(length);
        int n=v.getNumSequences();
        System.out.println("motif length="+length+".  possible seq ="+n);

        int actualTotal=0;
        int i;
        for (i=0; i<seq.length; i++)
        {
            actualTotal+=seq[i].length()-length+1;
            int d=seq[i].getDataIndex();
            float val=data.num(d, column);
            v.include(seq[i].getSequence(), val);
            if (i%200==0)
            {
                System.out.print(".");
            }
        }
        
        System.out.println("\n\nDone.");
    }
}

    
