package x.viz.seq;

import x.awt.*;
import x.label.*;
import java.awt.*;
import java.awt.event.*;

public class SequenceListViewer extends BufferPanel
{
    public static final String VIEW_CHANGE="focus changed";
    
    private SequencePixelRenderer renderer;
    private boolean mouseIsDown;
    private boolean isMetaDown;
    private Point mouse=new Point(0,0);
    private SequenceListModel model;
    private Rectangle focusRegion=new Rectangle(10,10,20,33);
    private int startX, startY;
    private Font labelFont=new Font("Helvetica", Font.PLAIN, 12);
    private Scrollbar verticalScroller=new Scrollbar(Scrollbar.VERTICAL);
    private Scrollbar horizontalScroller=new Scrollbar(Scrollbar.HORIZONTAL);
    
    public SequenceListViewer(SequenceListModel model)
    {
        this.model=model;
        setLayout(null);
        add(verticalScroller);
        //add(horizontalScroller);
        
        verticalScroller.addAdjustmentListener(new AdjustmentListener()
        {
            public void adjustmentValueChanged(AdjustmentEvent e)
            {
                setStartY(verticalScroller.getValue());
                sendViewChangeEvent();
            }
        }
        );
        
        addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent m)
            {
                handle(m,true);
            }
            public void mouseReleased(MouseEvent m)
            {
                handle(m,false);
            }
        });
        
        addMouseMotionListener(new MouseMotionAdapter()
        {
            public void mouseDragged(MouseEvent m)
            {
                handle(m, true);
            }
        });
    }
    
    private int scrollW()
    {
        return 16;
    }
    
    private int right()
    {
        return getSize().width-scrollW();
    }
    
    public void setModel(SequenceListModel model)
    {
        this.model=model;
        makeNewRenderer(getSize().width, getSize().height);
        redraw();
    }
    
    public void setStartY(int startY)
    {
        if (startY==this.startY) return;
        this.startY=startY;
        redraw();
    }
    
    public Rectangle getFocusRegion()
    {
        Rectangle r=new Rectangle(focusRegion);
        r.y+=startY;
        return r;
    }
    
    public void setBounds(int x, int y, int w, int h)
    {
        makeNewRenderer(w,h);
        int m=model.size();
        //System.out.println("h="+h+", model size="+m);
        if (h<m)
        {
            verticalScroller.setBounds(w-scrollW(),0,scrollW(),h);
            verticalScroller.setValues(startY, h-h*h/m, 0, m-h);
            verticalScroller.setVisible(true);
        }
        else
            verticalScroller.setVisible(false);
        super.setBounds(x,y,w,h);
    }
    
    void makeNewRenderer(int w, int h)
    {
        renderer=new SequencePixelRenderer(w-scrollW(), h, model.getSequences(), model.getColorScale());
    }
    
    void handle(MouseEvent m, boolean mouseIsDown)
    {
        boolean mouseChanged=(mouseIsDown!=this.mouseIsDown);
        this.mouseIsDown=mouseIsDown;
        isMetaDown= 0 != (m.getModifiers() & m.BUTTON3_MASK);
        mouse.x=m.getX();
        mouse.y=m.getY();
        if (mouseIsDown)
        {
            setFocusCorner(mouse.x, mouse.y);
            sendViewChangeEvent();
        }
        if (mouseChanged || mouseIsDown)
            repaint();
    }
    
    private void sendViewChangeEvent()
    {
        fireActionEvent(new ActionEvent(this, 0, VIEW_CHANGE));
    }
    
    private void setFocusCorner(int x, int y)
    {
        focusRegion.x=x-focusRegion.width;
        focusRegion.y=y-focusRegion.height;
    }
    
    public void paint(Graphics g)
    {
        super.paint(g);
        if (!focusRegionVisible())
            return;

        int x=focusRegion.x;
        int y=focusRegion.y;
        int w=right();
        int h=getSize().height;
        int fw=focusRegion.width;
        int fh=focusRegion.height;
        g.setColor(Color.black);
        outline(g,x-2,y-2,fw+4,fh+4,w,h);
        g.setColor(Color.white);
        outline(g,x-1,y-1,fw+2,fh+2,w,h);
        
        int a=startX+x;
        int b=startY+y;
        String xLabel="x: "+a+" to "+(a+fw-1);
        String yLabel="y: "+b+" to "+(b+fh-1);
        //String valueLabel="value at: "+a+", "+b+" = "+model.getSequences()[b].data(a);
        String[] label={xLabel};//, yLabel};//, valueLabel};
        BlockLabel.simpleLabel(g, label, labelFont, focusRegion,
            0,0,w,h);
    }
    
    private void outline(Graphics g, int x, int y, int fw, int fh, int w, int h)
    {
        /*
        g.drawLine(x,0,x,h);
        g.drawLine(0,y,w,y);
        g.drawLine(x+fw,0,x+fw,h);
        g.drawLine(0,y+fh,w,y+fh);
        */
        g.drawRect(x,y,fw,fh);
    }
    
    private boolean focusRegionVisible()
    {
        return focusRegion.x+focusRegion.width>=0;
    }
    
    private void hideFocusRegion()
    {
        focusRegion.x=-focusRegion.width-1;
    }
    
    protected synchronized void drawOffscreen(Graphics g)
    {
        //new Exception().printStackTrace();
        if (renderer!=null)
        {
            Image image=renderer.makeImage(startY,getSize().height);
            g.drawImage(image,0,0,null);
            //System.out.println("just drew image.");
            //System.out.println("image.width="+image.getWidth(null));
            //renderer.time(0,getSize().height);
        }
        else
        {
            System.out.println("no renderer");
            g.setColor(Color.black);
            g.drawString("No renderer yet.", 10, 30);
        }
    }
}
