package x.viz.seq;

import x.data.seq.*;
import x.awt.*;
import java.awt.*;
import java.awt.image.*;

public class SequencePixelRenderer
{
    int width, height;
    Sequence[] seq;
    byte[] pixels;
    IndexColorModel model;
    
    public SequencePixelRenderer(int width, int height, Sequence[] seq, ColorScale scale)
    {
        this.width=width;
        this.height=height;
        this.seq=seq;
                
        pixels=new byte[width*height];

        // make palette for indexed color model:
        byte[] r=new byte[256];
        byte[] g=new byte[256];
        byte[] b=new byte[256];
        for (int i=0; i<256; i++)
        {
            Color c=scale.color(i);
            r[i]=(byte)c.getRed();
            g[i]=(byte)c.getGreen();
            b[i]=(byte)c.getBlue();
        }
        
        model=new IndexColorModel(8,256,r,g,b);
    }
    
    public Image makeImage(int start, int length)
    {
        fill(start, length);
        Image image= Toolkit.getDefaultToolkit().createImage(
            new MemoryImageSource(width, height, model, pixels, 0, width));

        return image;
    }
    
    private void fill(int start, int length)
    {
        clear();
        int end=start+length;
        int p=0;
        for (int i=start; i<end; i++)
        {
//            Sequence s=seq[i];
            byte[] b=seq[i].data();
            int m=Math.min(b.length, width);
            System.arraycopy(b,b.length-m,pixels,p,m);
            p+=width;
        }
    }

    private void clear()
    {
        for (int i=pixels.length-1; i>=0; i--)
            pixels[i]=0;
    }

    public void time(int start, int length)
    {
        int n=100;
        int i;

        long s=System.currentTimeMillis();
        for (i=0; i<n; i++)
            fill(start, length);
        long fillTime=(System.currentTimeMillis()-s)/n;
        System.out.println("image time: "+fillTime);
    }
    
}