package x.viz.seq.misc;

import x.data.*;
import x.data.seq.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class MotifHistogram extends Frame implements ItemListener
{
    
    DataArray data;
    AnnotatedSequence[] seq;
    Sequence target=new Sequence("AATTTT");
    TextField entry;
    Choice valueChoice;
    ActionListener repainter=new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                repaint();
            }
        };
    
    public void itemStateChanged(ItemEvent e)
    {
        if (e.getSource()==valueChoice)
        {
            repaint();
        }
    }
    
    public static void main(String[] args)
    {
        if (args.length!=2)
        {
            System.err.println("usage: MotifHistogram metafile seqfile");
            return;
        }
        
        
        try
        {
            MotifHistogram c=new MotifHistogram();
            c.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {System.exit(0);}
            });
            c.setBounds(50,50,600,600);
            c.init(args[0], args[1]);
            c.setVisible(true);
        }
        catch (Exception e)
        {
            e.printStackTrace(System.err);
        }
    }
    
    
    
    public MotifHistogram()
    {
        super("Motif Histogram");
    }
    
    
    public void init(String metaFile, String seqFile) throws Exception
    {
        int i;
        
        entry=new TextField();
        add(entry);
        setLayout(null);
        entry.setBounds(5,30,100,25);
        entry.setText("AATTTT");
        entry.addActionListener(repainter);
        
        // Read data.
        data=new DataArray(metaFile);
        
        // Read basic sequences.
        Sequence[] unordered=FASTA.readFromFile(seqFile);
        
        // Translate to marked sequences.
        AnnotatedSequence[] u=UTR.translate(unordered);
        
        
        // Make AnnotatedSequence table.
        Hashtable seqTable=new Hashtable();
        for (i=0; i<u.length; i++)
            seqTable.put(u[i].getID(), u[i]);
        
        // Make averages.
        int goodcount=0;
        seq=new AnnotatedSequence[data.size()];
        
        // Correlate with data.
        for (i=0; i<data.size(); i++)
        {
            DataRow d=data.getRow(i);
            StringTokenizer t=new StringTokenizer(d.getText()[1]);
            String id=t.nextToken();
            AnnotatedSequence m=(AnnotatedSequence)seqTable.get(id);
            if (m!=null)
            {
                m.setDataIndex(i);
                seq[goodcount]=m;
                goodcount++;
            }
        }
        
        AnnotatedSequence[] temp=seq;
        seq=new AnnotatedSequence[goodcount];
        System.arraycopy(temp,0,seq,0,goodcount);
        System.out.println("processed AnnotatedSequences: good="+goodcount+" out of "+u.length);
        
        // Add value choice.
        valueChoice=new Choice();
        DataField[] field=data.getFields();
        int count=1;
        for (i=0; i<field.length; i++)
        {
            int type=field[i].getType();
            String name=field[i].getName();
            if (type==MetaData.NUMBER && name.trim().length()>0)
            {
                name=count+". "+name;
                valueChoice.addItem(name);
                count++;
            }
        }
        add(valueChoice);
        valueChoice.addItemListener(this);
        valueChoice.setBounds(150,30,215,20);
        
    }
    
    public void paint(Graphics g)
    {
        int dy=60;
        int dx=10;
        String targetName=entry.getText();
        target=new Sequence(targetName);
        int len=target.length();
        int column=valueChoice.getSelectedIndex();
        g.setColor(Color.black);
        g.fillRect(0,0,600,600);
        g.setColor(Color.gray);
        g.drawRect(dx,dy,500,500);
        int zero=250+dy;
        g.drawLine(dx,zero,500+dx,zero);
        g.setColor(Color.white);
        g.drawString(targetName, dx+5, dy+20);
        
        int bn=10;
        double[] valbin=new double[bn];
        int[] freqbin=new int[bn];
        
        
        for (int i=0; i<seq.length; i++)
        {
            Sequence s=seq[i].getSequence();
            int slen=s.length()-len;
            int d=seq[i].getDataIndex();
            float val=data.getRow(d).getNumbers()[column];
            for (int x=0; x<slen; x++)
            {
                if (s.match(target,x,0,len))
                {
                    //freq[x/50]++;
                    valbin[x/50]+=val;
                }
            }
        }
        
        
        
    }
    
}

    
